/**
 *	@file we_window_info.h
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2013
 *
 *	@brief This file contains the definitions needed for window managing among different applications.
 *
 *
 */

#ifndef WE_WINDOW_INFO_H_
#define WE_WINDOW_INFO_H_

#include <string>

namespace WE
{


enum WEApplnWindowState
{
	WE_WIN_NO_STATE = 0,
	WE_WIN_HIDDEN,
	WE_WIN_SHOWING,
	WE_WIN_VISIBLE,
	WE_WIN_HIDING,
	WE_WIN_RAISING,
	WE_WIN_LOWERING,
	WE_WIN_MOVING,
	WE_WIN_CHANGING_GEOMETRY,
	WE_WIN_DESTROYED
};



typedef struct WEWinInfo
{
	WEWinInfo( )
	{
		init();
	}

	void init()
	{
		this->winID		= -1;			//!< Unique ID given to each window
		this->windowState	= WE_WIN_NO_STATE;	//!< This defines the current state of the Application window
		this->identity		= "";			//!< This provides the name of the application executable
		this->applicationName	= "noAppName";		//!< Name of the Application to launch
		this->applicationPath	= "noAppPath";		//!< Path of the Application to launch
		this->applicationArgs	= "noAppArgs";		//!< Arguments of the Application to launch
		this->processID		= -1;
		this->processName	= "noProcessName";
		this->appID		= -1;
	};


	bool operator==(const WEWinInfo &p_c) const
	{	// we consider two instances equal if only the IDs match - the rest is considered additional information
		// std::cout << "operator==(const weAppInfo" << std::endl;
		return	   (p_c.winID	== this->winID)
			&& (p_c.processID	== this->processID)
			&& (p_c.appID	== this->appID);
	}


	// Members required for WM
	int winID;				//!< Unique ID given to each window
	WE::WEApplnWindowState windowState;	//!< This defines the current state of the Application window
	std::string identity;			//!< This provides the name of the application executable
	std::string applicationName;		//!< Name of the Application to launch
	std::string applicationPath;		//!< Path of the Application to launch
	std::string applicationArgs;		//!< Arguments of the Application to launch
	int processID;
	std::string processName;
	int appID;
} WEWindowInfo;
}

#endif /* WE_WINDOW_INFO_H_ */
