/*
 *	we_incident.h
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 *	@brief Based on WEEvent and intended to communicate from operator input/drawing (slow thread) to hardware and data maintenance (fast thread).
 *
 *	Some instances are created and directly handled in 'fast thread', mainly to trigger macros.
 *	Intended for 'internal' events while WESignal targets 'external' events to be handled in the same context.
 *
 *	@see WESignal
 */

#ifndef SRC_EVENT_HANDLING_INCLUDE_WE_INCIDENT_H_
#define SRC_EVENT_HANDLING_INCLUDE_WE_INCIDENT_H_

#include <cstring>
#include "we_event.h"

namespace WE
{

// SET_OBJECT_PROPERTY_EVENT - unused since 2013
//struct setObjectPropertyParameter
//{
//	// use WEEvent::value: union
//	//{
//	//	bool boolean;		//!< the value as boolean
//	//} value;
//	// use WEEvent::uvalue: int32 propertyID;		//!< the property ID
//	// use WEEvent::object: void *object;			//!< pointer to the object
//};



///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////// Internal events to trigger actions on ///////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// SCREEN_INIT_EVENT
//struct screenInitParameter
//{
//	// empty
//};
// SCREEN_EXIT_EVENT
//struct screenExitParameter
//{
//	// empty
//};

// SCREEN_REPEAT_EVENT
//struct screenRepeatParameter
//{
//	// empty
//};

// APPLICATION_INIT_EVENT
//struct applicationInitParameter
//{
//	// empty
//};

// APPLICATION_EXIT_EVENT
//struct applicationExitParameter
//{
//	// empty
//};

// APPLICATION_REPEAT_EVENT
//struct applicationRepeatParameter
//{
//	// empty
//};

// CHANGE_ATTRIBUTE_EVENT
//struct changeAttributeParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};

// CHANGE_BACKGROUND_COLOR_EVENT
//struct changeBackgroundColorParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};

// CHANGE_END_POINT_EVENT
//struct changeEndpointParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};

// SHOW_EVENT
//struct showParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};
// HIDE_EVENT
//struct hideParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};

// ENABLE_EVENT
//struct enableParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};

// DISABLE_EVENT
//struct disableParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};

// CHANGE_CHILD_LOCATION_EVENT
//struct changeChildLocationParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};

// CHANGE_CHILD_POSITION_EVENT
//struct changeChildPositionParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};

// CHANGE_SIZE_EVENT
//struct changeSizeParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};

// DELETE_OBJECT_POOL_CONFIRM_EVENT
// REMOVE_POOL_WIDGETS_CONFIRM_EVENT
// This is used as a round trip token sent from slow to fast and back to slow thread to confirm that pool can be erased
// i.e. that no more events for the pool are queued
// Note: For the roundtrip this has to become a 'WETrigger' on the way back.
//struct objectPoolConfirmParameter
//{
//	// use WEEvent::pointer: void * pointer;		//!< pointer to the pool that should be deleted or the ecu info where a pool should be merged
//	// use WEEvent::object: void * poolToActivate;	//!< the pool to make active
//	// use WEEvent::value: int32 loop_count;	//!< if this counter reaches zero, the pool can be deleted.
//				// Some events are currently moving from slow to fast to slow thread queue, so
//				// we need at least to send this event two rounds around fast and slow thread
//				// queue to be sure, that all events for a pool are processed. If the counter reaches
//				// zero, the pool can be deleted
//};

// RECREATE_AUX_SCREEN_DDOS_EVENT
//struct auxPoolParameter
//{
//	// use WEEvent::pointer: void *pool;
//	// use WEEvent::object: void *keepTempAssignmentsFrom;
//};

// SHOW_ALARM_IN_SETTINGS_PROJECT
//struct showAlarmInSettingsProjectParameter
//{
//	// use WEEvent::pointer: std::string * deadWSIconPath;
//	// use WEEvent::object: std::string * deadAuxInputUnitIconPath;
//	// use WEEvent::widget: std::string * deadAuxInputIconPath;
//};

// CHANGE_VALUE_EVENT
//struct changeValueParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};

// CHANGE_PRIORITY_EVENT
//struct changePriorityParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};

// CHANGE_ACTIVE_DATAMASK_EVENT
// this also exists as WETrigger to trigger drawing
//struct changeActiveDatamaskParameter
//{
//	// use WEEvent::object: void *currentDatamask;	//!< current datamask
//	// use WEEvent::widget: void *newDatamask;	//!< the new datamask to be changed to
//	// use WEEvent::pointer: void *pool;		//!< reference to the pool belonging to the event
//};

// CHANGE_SOFTKEY_MASK_EVENT
// this also exists as WETrigger to trigger drawing

// WORKING_SET_ACTIVE_EVENT
//struct workingSetActiveParameter
//{
//	// use WEEvent::pointer: void *pool;	//!< the pool
//};

// WORKING_SET_DEACTIVE_EVENT
//struct workingSetDeactiveParameter
//{
//	// use WEEvent::pointer: void *pool;	//!< the pool
//};

// CHANGE_FONT_ATTRIBUTES_EVENT
//struct changeFontAttributesParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};

// CHANGE_LINE_ATTRIBUTES_EVENT
//struct changeLineAttributesParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};

// CHANGE_FILL_ATTRIBUTES_EVENT
//struct changeFillAttributesParameter
//{
//	// use WEEvent::object: void *object;	//!< the object referenced with the event
//};




union incidentParameter
{
};




struct WEIncident:virtual WEEvent
{

	union incidentParameter para;	// currently empty

	/**
	 *	@brief Constructor directly defining the event type - UNKNOWN_EVENT if none provided
	 *
	 *	@param p_type The type of event to be created
	 *	@param p_class Keep default - is only modified by classes inheriting this one
	 *
	 *	@warning Logically p_type should clearly define what event class to generate but nothing is enforced there.
	 *
	 *	Marked as 'explicit' i.e. not allowed for implicit casting.
	 */
	explicit WEIncident(WEEventType p_type = UNKNOWN_EVENT, WEEventClass p_class = WE_INCIDENT)
	:WEEvent(p_type, p_class)
	{
		std::memset(&(this->para), 0, sizeof(this->para));
	}

	virtual ~WEIncident()
	{
		// nothing to clean up
	}


// unused since 2013:
//	inline void setSetObjectPropertyParameter(bool p_value, int32 p_propertyID, void * const p_object)
//	{
//		this->value = (int32) p_value;	//!< the value as boolean
//		this->uvalue = p_propertyID;	//!< the property ID
//		this->object = p_object;	//!< pointer to the object
//	}


	inline void setChangeAttributeParameter(void * const p_object)
	{
		this->object = p_object;	//!< the object referenced with the event
	}

	inline void setChangeBackgroundColorParameter(void * const p_object)
	{
		this->object = p_object;	//!< the object referenced with the event
	}

	inline void setChangeEndpointParameter(void * const p_object)
	{
		this->object = p_object;	//!< the object referenced with the event
	}

	inline void setShowParameter(void * const p_object)
	{
		this->object = p_object;	//!< the object referenced with the event
	}

	inline void setHideParameter(void * const p_object)
	{
		this->object = p_object;	//!< the object referenced with the event
	}

	inline void setEnableParameter(void * const p_object)
	{
		this->object = p_object;	//!< the object referenced with the event
	}

	inline void setDisableParameter(void * const p_object)
	{
		this->object = p_object;	//!< the object referenced with the event
	}

	inline void setChangeChildLocationParameter(void * const p_object)
	{
		this->object = p_object;	//!< the object referenced with the event
	}

	inline void setChangeChildPositionParameter(void * const p_object)
	{
		this->object = p_object;	//!< the object referenced with the event
	}

	inline void setChangeSizeParameter(void * const p_object)
	{
		this->object = p_object;	//!< the object referenced with the event
	}

	inline void setObjectPoolConfirmParameter(	void * const p_pointer,
							void * const p_poolToActivate)
	{
		this->pointer	= p_pointer;		//!< pointer to the pool that should be deleted or the ecu info where a pool should be merged
		this->object	= p_poolToActivate;	//!< the pool to make active
		this->value	= 2;			// magic number: after two round trips all events for this pool should be handled, then we can delete it
	}

	inline void setShowAlarmInSettingsProjectParameter(std::string * p_deadWSIcon, std::string * p_deadAuxInputUnitIcon = nullptr, std::string * p_deadAuxInputIcon = nullptr)
	{
		this->pointer	= p_deadWSIcon;
		this->object	= p_deadAuxInputUnitIcon;
		this->widget	= p_deadAuxInputIcon;
	}

	inline void setChangeValueRequestParameter(unsigned short p_index, unsigned short p_subIndex, int p_newValue)
	{
		return setChangeValueRequestParameter( ((((unsigned int)p_index) << 16) | p_subIndex), p_newValue);
	}

	// trigger variable change e.g. from navigation
	inline void setChangeValueRequestParameter(unsigned int p_combinedIndex, int p_newValue)
	{
		this->uvalue = p_combinedIndex;	//!< Variable's index + subIndex
		this->value = p_newValue;
	}

	inline void setChangeValueParameter(void * const p_object)
	{
		this->object = p_object;	//!< the object referenced with the event
	}

	inline void setChangePriorityParameter(void * const p_object)
	{
		this->object = p_object;	//!< the object referenced with the event
	}

	inline void setWorkingSetActiveParameter(void * const p_pool)
	{
		this->pointer = p_pool;	//!< the pool
	}

	inline void setWorkingSetDeactiveParameter(void * const p_pool)
	{
		this->pointer = p_pool;	//!< the pool
	}

	inline void setChangeFontAttributesParameter(void * const p_object)
	{
		this->object = p_object;		// nobody cares ;-)
	}

	inline void setChangeLineAttributesParameter(void * const p_object)
	{
		this->object = p_object;		// nobody cares ;-)
	}

	inline void setChangeFillAttributesParameter(void * const p_object)
	{
		this->object = p_object;		// nobody cares ;-)
	}
};



/**
 *	@brief Event listener interface that needs to be implemented to get notified of input events.
 *
 *	Subscription to events has to be implemented separately but they are expected to be handled in
 *	'incidentOccurred()'.
 */
class WEIncidentListener
{
public:
	/**
	 *	@brief Handle events in this method.
	 *
	 *	@param[in] p_incident The event that has occurred.
	 *
	 *	@returns an error code - WE_SUCCESS in case the event was successfully handled.
	 *
	 *	p_incident should potentially be completed in this call but it should be carefully considered
	 *	to e.g. alter the event type since the event may be forwarded to other entities
	 *	after this method terminated. They are probably expecting the original content then.
	 *	I.e. generally p_incident should be copied before the existing content is modified.
	 *
	 *	It is suggested to implement final event handling in the data context (fast thread) in this
	 *	method.
	 *	If they need handling in GUI context (slow thread) that should have happened before
	 *	they were considered 'WEIncident' only.
	 *
	 */
	virtual int incidentOccurred(WEIncident & p_incident) = 0;
	virtual ~WEIncidentListener() = default;
};
}	// namespace WE

#endif /* SRC_EVENT_HANDLING_INCLUDE_WE_INCIDENT_H_ */
