/*
 *	we_navigation_event.h
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 *	@brief Based on WEEvent and intended for 'actions' triggered by operator input or command.
 *
 *	Wherever these are triggered now there should be a configurable option in the future.
 *
 *	Navigation events are typically created in navigation scheme, sent to the widget controller
 *	(which may partly handle them) and then forwarded to the appropriate DDO (which could be the
 *	one touched, the one with focus or the inplace editor for the one with focus).
 *
 *	The DDO should never return the event to the same widget controller to avoid infinite recursion.
 */

#ifndef SRC_EVENT_HANDLING_INCLUDE_WE_NAVIGATION_EVENT_H_
#define SRC_EVENT_HANDLING_INCLUDE_WE_NAVIGATION_EVENT_H_


#include "we_input_event.h"

namespace WE
{

// ESCAPE_EVENT
//struct escapeParameter
//{
//	// use WEEvent::value: std::int32_t origin;		//!< where the event was triggered
//};

// only triggered if the last alarm is confirmed via ESC it seems:
//JUMP_TO_PAGE_EVENT
//struct jumpToPageParameter
//{
//	// empty
//};

//OPEN_FOR_INPUT_EVENT
// special event for some corner cases in aux assignment and pointer DDO it seems - otherwise created by navigation scheme
//struct openForInputParameter
//{
//	// use WEEvent::value: std::int32_t origin;		//!< where the event was triggered (EVENT_ORIGIN_OPERATOR or EVENT_ORIGIN_SYSTEM)
//	// use WEEvent::object: void *object;		//!< trigger action of this object in fast thread
//	// use WEEvent::controller: void *controller;	//!< filled by the widget controller
//};

// ENABLED_WITH_FOCUS_EVENT
//struct focusGainedParameter
//{
//	// use WEEvent::value: std::int32_t value;		//!< the value as integer
//	// use WEEvent::uvalue: std::uint32_t oldState;	//!< border state before focus gained (mainly to figure out if it was "open for input")
//	// use WEEvent::object: void *object;		//!< internal used by iso-vt only
//	// use WEEvent::controller: void *controller;	//!< filled by the widget controller
//};

//FOCUS_LOST_EVENT
// created by navigation scheme
//struct focusLostParameter
//{
//	// use WEEvent::value: std::int32_t origin;		//!< where the event was triggered (EVENT_ORIGIN_OPERATOR or EVENT_ORIGIN_SYSTEM)
//	// use WEEvent::object: void *object;		//!< trigger action of this object in fast thread
//	// use WEEvent::controller: void *controller;	//!< filled by the widget controller
//};

//ENABLED_OUTPUT_EVENT
// created by navigation scheme
//struct enabledOutputParameter
//{
//	// use WEEvent::value: std::int32_t origin;		//!< where the event was triggered (EVENT_ORIGIN_OPERATOR or EVENT_ORIGIN_SYSTEM)
//	// use WEEvent::controller: void *controller;	//!< filled by the widget controller
//};

//DISABLED_EVENT
// created by navigation scheme
//struct disabledParameter
//{
//	// use WEEvent::value: std::int32_t origin;	//!< where the event was triggered (EVENT_ORIGIN_OPERATOR or EVENT_ORIGIN_SYSTEM)
//};

//CHANGE_EDITED_VALUE_EVENT
// This event was the same as ENCODER_MOVE_EVENT
// created by navigation scheme
//struct changeEditedValueParameter
//{
//	// use the sign of 'value' for that: std::int32_t direction;	//!< the rotation direction of the encoder
//	// use WEEvent::value: std::uint32_t steps;		//!< number of steps the encoder was rotated, sign is the direction
//	// use WEEvent::uvalue: std::uint32_t dynamicSteps;	//!< number of dynamic steps belonging to steps the encoder was rotated
//	// use WEEvent::controller: void *controller;	//!< filled by the widget controller
//	// use WEEvent::widget != nullptr: bool triggeredByScrollbar;
//};

//CONFIRM_EDITED_VALUE_EVENT originates without value which is set by DDO before using it for action
// created by navigation scheme and
//	app_boolean_ddo.cpp - to skip 'open for input'
//	app_int_ddo.cpp - if sendValueDirectly and to unify CONFIRM_INPLACE_VALUE_EVENT
//	app_string_ddo.cpp - to unify CONFIRM_INPLACE_VALUE_EVENT
//CONFIRM_INPLACE_VALUE_EVENT always has the value set, typically converted to CONFIRM_EDITED_VALUE_EVENT by DDO
// set in nav_object.cpp - in confirmEditedValue(int p_intValue, void * p_stringValue)
//struct confirmEditedValueParameter
//{
//	//union
//	//{
//		// use WEEvent::value: std::int32_t integer;		//!< value as integer
//		// use WEEvent::pointer: std::string *string;	//!< value as string pointer
//		// use WEEvent::pointer: void *typeless;		//!< type less value
//	//} value;			//!< the value
//	// use WEEvent::object: void *object;			//!< internal used, by iso-vt only
//	// use WEEvent::controller: void *controller;		//!< filled by the widget controller
//	// user WEEvent::uvalue: bool onlyChanged;		//!< To be used when Confirm edited value is misused for
//					//send value directly change events
//};

//CONFIRM_NEW_EDITED_VALUE_EVENT
// set
//	app_int_ddo.cpp
//	app_string_ddo.cpp
//struct confirmNewEditedValueParameter
//{
//	// empty
//};


//TOOLKIT_WINDOW_ACTIVE_EVENT
// set in we_widget.cpp and only evaluated in the VT it seems (PClient only has one window)
//struct windowActiveParameter
//{
//	// empty
//};

//TOOLKIT_WINDOW_DEACTIVE_EVENT
//struct windowDeactiveParameter
//{
//	// empty
//};



union navigationEventParameter
{
};



// Navigation event inherits 'WEInputEvent' in case navigation decides to forward the input event to DDO for evaluation.
// Thus it also inherits 'WEIncident' as it's often used to transport effect of user input to fast thread.
struct WENavigationEvent:virtual WEInputEvent
{

	union navigationEventParameter naviPara;	// currently empty (as for WEIncident)

	/**
	 *	@brief Constructor directly defining the event type - UNKNOWN_EVENT if none provided
	 *
	 *	@param p_type The type of event to be created
	 *	@param p_class Keep default - is only modified by classes inheriting this one
	 *
	 *	@warning Logically p_type should clearly define what event class to generate but nothing is enforced there.
	 *
	 *	Marked as 'explicit' i.e. not allowed for implicit casting.
	 */
	explicit WENavigationEvent(WEEventType p_type = UNKNOWN_EVENT, WEEventClass p_class = WE_NAVIGATION_EVENT)
	:WEEvent(p_type, p_class),	// otherwise the empty constructor is called
	 WEIncident(p_type, p_class),
	 WEInputEvent(p_type, p_class)
	{
		memset(&(this->naviPara), 0, sizeof(this->naviPara));
	}

	/*
	 *	Copy constructor to convert an input event into a navigation event.
	 */
	WENavigationEvent(const WEInputEvent & p_event)
	: WEEvent(p_event),
	  WEIncident(p_event),
	  WEInputEvent(p_event)
	{
		memset(&(this->naviPara), 0, sizeof(this->naviPara));
		this->eventClass = WE_NAVIGATION_EVENT;
	}

	virtual ~WENavigationEvent()
	{
		// nothing to clean up
	}


	inline void setEscapeParameter(std::int32_t p_origin, void * const p_object = nullptr)
	{
		this->value  = p_origin;		//!< where the event was triggered (auto or by user input)
		this->object = p_object;
	}

// unused:
//	inline void setChangeInplaceValueParameter(std::int32_t p_value)
//	{
//		this->value   = p_value;	//!< the value as integer
//		this->pointer = nullptr;	//!< the value as string pointer
//	}

// unused:
//	inline void setChangeInplaceValueParameter(std::string * const p_value)
//	{
//		this->value   = -1;
//		this->pointer = p_value;
//	}

	inline void setChangeInplaceValueParameter(void * const p_value)
	{
		// this->value   = (std::int32_t)p_value;	// PC: error: cast from 'void*' to 'WE::std::int32_t' {aka 'int'} loses precision [-fpermissive]
		this->pointer = p_value;
	}

// all three unused - AppWidgetController generates the event for the DDO but never adds any content
//	inline void setEscapeInplaceEditParameter(std::int32_t p_value, void * const p_object = nullptr)
//	{
//		eventParameter.escapeInplaceEdit.value.integer = p_value;
//		eventParameter.escapeInplaceEdit.isInteger = true;
//		eventParameter.escapeInplaceEdit.isString = false;
//		eventParameter.escapeInplaceEdit.object = p_object;
//	}
//
//
//	inline void setEscapeInplaceEditParameter(std::string * const p_strValue, void * const p_object = nullptr)
//	{
//		eventParameter.escapeInplaceEdit.value.string = p_strValue;
//		eventParameter.escapeInplaceEdit.isString = true;
//		eventParameter.escapeInplaceEdit.isInteger = false;
//		eventParameter.escapeInplaceEdit.object = p_object;
//	}
//
//	inline void setEscapeInplaceEditParameter(void * const p_value, void * const p_object = nullptr)
//	{
//		eventParameter.escapeInplaceEdit.value.typeless = p_value;
//		eventParameter.escapeInplaceEdit.isInteger = false;
//		eventParameter.escapeInplaceEdit.isString = false;
//		eventParameter.escapeInplaceEdit.object = p_object;
//	}

	inline void setOpenForInputParameter(std::int32_t p_origin, void * const p_object = nullptr)
	{
		this->value	= p_origin;	//!< where the event was triggered (EVENT_ORIGIN_OPERATOR or EVENT_ORIGIN_SYSTEM)
		this->object	= p_object;	//!< trigger action of this object in fast thread
		// leave this->controller untouched here - it's filled by the widget controller
	}

	inline void setFocusGainedParameter(std::int32_t p_origin, std::uint32_t p_oldState = 0, void * const p_object = nullptr)
	{
		this->value	= p_origin;	//!< where the event was triggered (EVENT_ORIGIN_OPERATOR or EVENT_ORIGIN_SYSTEM)
		this->uvalue	= p_oldState;	//!< border state before focus gained (mainly to figure out if it was "open for input" ... but seems nobody checks that)
		this->object	= p_object;	//!< internal used by iso-vt only
		// leave this->controller untouched - it's filled by the widget controller
	}

	inline void setFocusLostParameter(std::int32_t p_origin, void * const p_object = nullptr)
	{
		this->value	= p_origin;		//!< where the event was triggered (EVENT_ORIGIN_OPERATOR or EVENT_ORIGIN_SYSTEM)
		this->object	= p_object;		//!< trigger action of this object in fast thread
		// leave this->controller untouched here - it's filled by the widget controller
	}

	inline void setEnabledOutputParameter(std::int32_t p_origin)
	{
		this->value	= p_origin;	//!< where the event was triggered (EVENT_ORIGIN_OPERATOR or EVENT_ORIGIN_SYSTEM)
		// leave this->controller untouched here - it's filled by the widget controller
	}

	inline void setDisabledParameter(std::int32_t p_origin)
	{
		this->value = p_origin;	//!< where the event was triggered (EVENT_ORIGIN_OPERATOR or EVENT_ORIGIN_SYSTEM)
		// leave this->controller untouched here - it's filled by the widget controller
	}

	inline void setChangeEditedValueParameter(std::int32_t p_steps, std::uint32_t p_dynamicSteps, void * p_triggeredByThisScrollbar = nullptr)
	{
		this->value	= p_steps;			//!< number of steps the encoder was rotated, sign indicates direction
		this->uvalue	= p_dynamicSteps;		//!< number of dynamic steps belonging to steps the encoder was rotated
		this->widget	= p_triggeredByThisScrollbar;	//!< scrollbar which created this event (if any)
		// leave this->controller untouched here - it's filled by the widget controller
	}

	inline void setConfirmEditedValueParameter(std::int32_t p_intValue, const std::string & p_stringValue = "", void * const p_object = nullptr, bool p_onlyChanged = false)
	{
		this->value	= p_intValue;		//!< value as integer
		this->pointer	= nullptr;	//!< value as string pointer
		this->strValue	= p_stringValue;	//!< value as string pointer
		this->object	= p_object;		//!< trigger action of this object in fast thread
		this->uvalue	= p_onlyChanged;	//!< To be used when Confirm edited value is misused for send value directly change events
		// leave this->controller untouched here - it's filled by the widget controller
	}
};


/**
 *	@brief Event listener interface that needs to be implemented to get notified of input events.
 *
 *	Subscription to events has to be implemented separately but they are expected to be handled in
 *	'inputEventOccurred()'.
 */
class WENavigationEventListener
{
public:
	/**
	 *	@brief Handle events in this method.
	 *
	 *	@param[in] p_event The event that has occurred.
	 *
	 *	@returns an error code - WE_SUCCESS in case the event was successfully handled.
	 *
	 *	p_event should potentially be completed in this call but it should be carefully considered
	 *	to e.g. alter the event type since the event may be forwarded to other entities
	 *	after this method terminated. They are probably expecting the original content then.
	 *	I.e. generally p_event should be copied before the existing content is modified.
	 *
	 *	It is suggested to use this method to handle navigation events in the GUI context (slow thread).
	 *	If they need (additional) handling in data context (fast thread) it should be implemented
	 *	in 'incidentOccurred(WEIncident)'
	 *
	 *	@see incidentOccurred(WEIncident)
	 */
	virtual int navigationEventOccurred(WENavigationEvent & p_event) = 0;
	virtual ~WENavigationEventListener() = default;
};
}	// namespace WE

#endif /* SRC_EVENT_HANDLING_INCLUDE_WE_NAVIGATION_EVENT_H_ */
