/**
 *	@file toolkit_we_region.h
 *
 *	@author TANMOY.DEB AT ARICENT.COM
 *	@author TANMOY DEB
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 *
 *	@brief This is the abstract class for the WERegion implementation.
 *
 *	This is an abstract class for the WERegion implementation.
 */

#ifndef TOOLKIT_WE_REGION_H_
#define TOOLKIT_WE_REGION_H_

namespace WE
{

class WERect;

/**
 *	@class ToolkitWERegion
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@brief Abstract interface for a region. Toolkit specific implementation.
 *
 *	@namespace WE
 */
class ToolkitWERegion
{
public:
	/**
	 *	@fn ~ToolkitWERegion()
	 *	@brief Destructor from the abstract region class
	 *
	 *	@author TANMOY.DEB AT ARICENT.COM
	 *	@author TANMOY DEB
	 *
	 *	ToolkitWERegion is a completely abstract class
	 *	Therefore it has no Constructor and only the virtual Destructor, which must be defined
	 */
	virtual ~ToolkitWERegion() = default;

	/**
	 *	@fn virtual WERect boundingRect() = 0
	 *
	 *	@brief Returns the bounding rectangle of this region.
	 *
	 *	@author TANMOY.DEB AT ARICENT.COM
	 *	@author TANMOY DEB
	 *
	 *	@returns The bounding rectangle of this region.
	 *
	 *	@see WERect.cpp
	 *
	 *	Returns the bounding rectangle of this region.
	 */
	virtual WERect boundingRect() = 0;

	/**
	 *	@fn virtual ToolkitWERegion* subtract ( ToolkitWERegion* p_toolkitWERegion ) = 0
	 *
	 *	@brief Returns a region which is subtracted from this region.
	 *
	 *	@author tanmoy.deb AT aricent.com
	 *	@author Tanmoy Deb
	 *
	 *	@param[in] p_toolkitWERegion The region to be substracted from the another region.
	 *
	 *	@returns The result region
	 *
	 */
	virtual ToolkitWERegion* subtract ( ToolkitWERegion* p_toolkitWERegion ) = 0;

	/**
	 *	@fn virtual ToolkitWERegion* united ( ToolkitWERegion* p_toolkitWERegion ) = 0
	 *
	 *	@brief Returns a region which is added with this region.
	 *
	 *	@author Tanmoy Deb
	 *	@author tanmoy.deb AT aricent.com
	 *
	 *
	 *	@param  p_toolkitWERegion
	 *
	 *	@returns ToolkitWERegion The result Region after doing the united operation.
	 */
	virtual ToolkitWERegion* united ( ToolkitWERegion* p_toolkitWERegion ) = 0;
};
}

#endif /* TOOLKIT_WE_REGION_H_ */
