/**
 *	@file we_painter_path.h
 *
 *	@author tanmoy.deb@aricent.com
 *	@author Tanmoy Deb
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 */
#ifndef WE_PAINTER_PATH_H_
#define WE_PAINTER_PATH_H_

#include <string>
#include "we_region.h"

namespace WE
{

class WEFont;
struct WEPoint;
class ToolkitWEPainterPath;

enum PainterPathFillRules
{
	FILL_RULE_ODD_EVEN,
	FILL_RULE_WINDING
};

class WEPainterPath
{
public:

	WEPainterPath(WERect &p_rect,WE::WERegionType p_regionType, int p_startAngle, int p_spanAngle, bool p_antialiasing);

	WEPainterPath();

	virtual ~WEPainterPath();

	/**
	 *	@fn ToolkitWEPainterPath* getToolkitWEPainterPath()
	 *
	 *	@author Tanmoy Deb
	 *	@author tanmoy.deb AT aricent.com
	 *
	 *
	 *	@returns The reference to the ToolkitWEPainterPath. That will be used to access the
	 *	QPainterPath from the WEPainterPath.
	 *
	 *	Get the toolkitWEPainterPath.
	*/
	ToolkitWEPainterPath* getToolkitWEPainterPath() const;

	/**
	 *	@fn void setToolkitWEPainterPath(ToolkitWEPainterPath *p_toolkitWEPainterPath)
	 *
	 *	@brief Set the toolkit PainterPath
	 *
	 *	@author tanmoy.deb AT aricent.com
	 *	@author Tanmoy Deb
	 *
	 *	@param[in] p_toolkitPainterPath The toolkit PainterPath
	 *
	 *	Set the ToolkitWEPainterPath.
	*/
	void setToolkitWEPainterPath(ToolkitWEPainterPath *p_toolkitWEPainterPath);

	WERect getBoundingRect() const;

	void init();

	WEPainterPath operator+ (const WEPainterPath &p_wePainterPath);

	WEPainterPath operator- (const WEPainterPath &p_wePainterPath);

	// The copy Constructor
	WEPainterPath (const WEPainterPath&  p_wePainterPath);

	WEPainterPath& operator=(const WEPainterPath& p_other);

	int getStartAngle() const;

	int getSpanAngle() const;

	void addText(WEFont &p_font, const std::string & p_text, int p_flag);
	void addEllipse (WERect & p_rect);
	void addPolygon (WEPoint * p_points, unsigned int p_numberOfPoints);

	void setFillRule(int p_rule);

private:

	// The toolkit WEPainterPath
	std::unique_ptr<ToolkitWEPainterPath> toolkitWEPainterPath;
	// The rectangle
	WERect rect;
	//The start angle
	int startAngle;
	//The end angle
	int spanAngle;
	//The region Type
	WE::WERegionType regionType;

	bool antialiasing;
};
}

#endif
