/**
 *	@file qt_we_multimedia.h
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2011
 *
 *	@brief Class definitions of MyVideoWidget and QtWEMultimediaSource
 */


#ifndef QT_WE_MULTIMEDIA_H_
#define QT_WE_MULTIMEDIA_H_

#include <tr1/unordered_map>
#include <QSoundEffect>

namespace WE {

class WEMultimediaPlayer;

class QtWEMultimediaSource : public QObject
{
	Q_OBJECT
public:
	/**
	 * Constructor. Initialize members with (hopefully) sane default values
	 * @param parent The QObject parent, can be omitted. Not really interesting for our application
	 */
	explicit QtWEMultimediaSource(WEMultimediaPlayer * p_player, QObject * parent = nullptr);

	/**
	 * Destructor. Frees memory and disconnects video and audio pathes.
	 * Does not delete the widgets! (WEWidget is deleted by standard WE ways and the phonon video
	 * widget SHOULD be deleted by Qt's automatic freeing because it is a child of another
	 * QWidget that gets destroyed)
	 */
	virtual ~QtWEMultimediaSource();

	void fireEvent(void);



private:
	friend class WEMultimediaPlayer;	//!< The multimedia player may access private members
	std::string path;	//!< The identifier of the media file (it's file path)
	unsigned char volume;	//!< The intended volume. What is really set is based on volume variables!
	int loopCount;	//!< How often this file has repeated already
	int maxLoops;	//!< How often shall this media file be repeated (-1 for infinite loop)
	unsigned int playbackGap;	//!< playback gap in milliseconds between each loop
	int state;	//!< The state of this media file as reported by phonon
	int oldState;
	int category;	//!< Either multimedia, alarm, event, etc. see enum MediaCategory in we_multimedia_player.h
	WEWidget * videoWidget;	//!< The WEWidget which's QWidget acts as a parent for our own video widget

	bool isVideo;

	QSoundEffect * mediaObject;	//!< The phonon media object that is used for controlling playback

	WEMultimediaPlayer * thePlayer;
};

typedef std::tr1::unordered_map<std::string, QtWEMultimediaSource*> sourceMap;	//!< Typedef of the hash map that contains all source files identified by it's file path

}  // namespace WE


#endif /* QT_WE_MULTIMEDIA_H_ */
