/**
 *	@file we_basic_rectangle_node.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 */

#ifndef WE_BASIC_RECTANGLE_NODE_H
#define WE_BASIC_RECTANGLE_NODE_H

#include "we_basic_geometry_node.h"
#include "../../common/include/we_rect.h"

namespace WE
{

/**
 *
 * This class is the base class for all rectangle nodes.
 *
 */
class WEBasicRectangleNode : virtual public WEBasicGeometryNode
{

public:
	~WEBasicRectangleNode() = default;
	void setRect(const WERect& p_r);
	void setSize(const QSize& p_s);
	void setPosition(const QPoint& p_p);
	void setX(int p_x);
	void setY(int p_y);
	void setWidth(int p_width);
	void setHeight(int p_height);

	WERect getRect() const
	{
		return this->rect;
	}

	QSize getSize() const
	{
		return QSize(getWidth(), getHeight());
	}

	QPoint getPosition() const
	{
		return QPoint(getX(), getY());
	}

	int getX() const
	{
		return rect.getX();
	}

	int getY() const
	{
		return rect.getY();
	}

	int getWidth() const
	{
		return rect.getWidth();
	}

	int getHeight() const
	{
		return rect.getHeight();
	}


protected:

	WERect rect{};

	WEBasicRectangleNode();
	explicit WEBasicRectangleNode(const WERect& p_rect);
};
} //End of namespace WE

#endif // WE_BASIC_RECTANGLE_NODE_H
