/**
 *	@file we_controller_node.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 */

#ifndef WE_CONTROLLER_NODE_H
#define WE_CONTROLLER_NODE_H

#include "we_base_node.h"

namespace WE
{
/**
 *
 *	This class can be used to control the visibility of other WENodes in the Qt quick scene graph.
 *
 *	For the software renderer to accept this node (and its subtree) it cannot just be a 'QSGNode'
 *	but has at least to be a 'QSGSimpleRectNode'
 *
 *	@TODO: Do we have to maintain the size of the rect then?
 */
class WEControllerNode : public QSGNode, public WEBaseNode
{
public:

	WEControllerNode();
	virtual ~WEControllerNode() = default;

	void draw(WEBaseNode *p_node);


	/**
	 *	@brief To be called before anything substantial is done in 'update[Paint]Node()'
	 */
	void begin();


	/**
	 *	@brief To be called after everything substantial is done in 'update[Paint]Node()'
	 */
	void end();


	bool isSubtreeBlocked() const override
	{
		return this->isNodeHidden();
	}

private:
	std::list<WEBaseNode*> nodes{};
	QSGNode * lastDrawnNode{nullptr};
};
} //End of namespace WE

#endif // WE_CONTROLLER_NODE_H
