/**
 *	@file we_image_node.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 */

#ifndef WE_IMAGE_NODE_H
#define WE_IMAGE_NODE_H

// Forward declarations:
namespace WE
{
class WEWidget;
class WEImageNode;
class WEControllerNode;
}

#include "we_basic_rectangle_node.h"
#include "we_node_image_support.h"

namespace WE
{


/**
 *
 * This class can be used to draw an image with the Qt quick scene graph.
 *
 */
class WEImageNode : public WEBasicRectangleNode//, public QObject
{

public:

	explicit WEImageNode(WEWidget *p_widget=nullptr);
	virtual ~WEImageNode();

	void setRotation(float p_rotation);
	void setTexture(QSGTexture *p_texture);
	void setImage(WEImage *p_image);
	void setSVGBGColor(const WEColor& p_svgBGColor);
	void updateGIF();
	void apply();

	float getRotation() const
	{
		return this->rotation;
	}

	QSGTexture * getTexture() const
	{
		return this->imageSupport.getTexture();
	}

	WEImage * getImage() const
	{
		return this->imageSupport.getImage();
	}

	WEColor getSVGBGColor() const
	{
		return this->svgBGColor;
	}

	static void updateNode(WEImageNode * & p_node, WEControllerNode *p_controller, WEImage *p_image, const WERect& p_rect, float p_rotation=0);

	static void updateNode(WEImageNode * & p_node, WEControllerNode *p_controller, WEWidget *p_widget, WEImage *p_image, const QPoint& p_position=QPoint(0,0), float p_rotation=0);

	static void updateNode(WEImageNode * & p_node, WEControllerNode *p_controller, WEWidget *p_widget, WEImage *p_image, const WERect& p_rect, float p_rotation=0, const WEColor& p_svgBGColor=WEColor::transparentColor);


protected:

	QSGGeometry *geometry;
	WENodeImageSupport imageSupport;
	float rotation;
	WEColor svgBGColor;

	/**
	 *	@fn void rotationHelper(QSGGeometry::TexturedPoint2D *p, int i, float x, float y, float tx, float ty, float cx, float cy, float s, float c)
	 *
	 *	@brief Apply the rotation to the point and write it with the texture coordinates to the result array
	 *
	 *	@author preths@topcon.com
	 *	@author Patrick Reths
	 *
	 *	@param[out] p result array
	 *	@param[in] i index in the result array
	 *	@param[in] x position of the point to rotate
	 *	@param[in] y position of the point to rotate
	 *	@param[in] tx texture position
	 *	@param[in] ty texture position
	 *	@param[in] cx center of the image on the x axis
	 *	@param[in] cy center of the image on the y axis
	 *	@param[in] s precalculated sin
	 *	@param[in] c precalculated cos
	 */
	void rotationHelper(QSGGeometry::TexturedPoint2D *p_p, int p_i, float p_x, float p_y, float p_tx, float p_ty, float p_cx, float p_cy, float p_s, float p_c);
};
} //End of namespace WE

#endif // WE_IMAGE_NODE_H
