/**
 *	@file we_line_node.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 */

#ifndef WE_LINE_NODE_H
#define WE_LINE_NODE_H

#include "we_geometry_node.h"

//#include <vector>

namespace WE
{
class WEControllerNode;

/**
 *
 * This class can be used to draw a line with the Qt quick scene graph.
 *
 */
class WELineNode : public WEGeometryNode
{
public:

	explicit WELineNode(MaterialType p_materialType);
	virtual ~WELineNode() = default;

	void setSize(unsigned int p_size);
	void setWidth(float p_width);
	void setPoints(const std::vector<QPointF>& p_points);
	void setPoint(unsigned int p_i, const QPointF& p_point);
	void apply();

	unsigned int getSize() const
	{
		return this->points.size();
	}

	float getWidth() const
	{
		return this->width;
	}

	const std::vector<QPointF>& getPoints() const
	{
		return this->points;
	}

	const QPointF& getPoint(unsigned int p_i) const
	{
		return this->points[p_i];
	}

	static void updateNode(WELineNode * & p_node, WEControllerNode *p_controller, const std::vector<QPointF>& p_points, float p_width, const WEColor& p_color);

	static void updateNode(WELineNode * & p_node, WEControllerNode *p_controller, const std::vector<QPointF>& p_points, float p_width, WEImage * p_image, const QSize & p_geometrySize);

protected:

	static void updateNode(WELineNode * & p_node, WEControllerNode *p_controller, const std::vector<QPointF>& p_points, float p_width, MaterialType p_materialType, const WEColor *p_color, WEImage * p_image, const QSize& p_geometrySize);

	void drawLine(unsigned int p_i, float p_width, const QPointF& p_p0, const QPointF& p_p1);
	void drawLine(unsigned int p_i, float p_width, float p_x0, float p_y0, float p_x1, float p_y1);

	std::vector<QPointF> points{};
	float width{1};
};
} //End of namespace WE

#endif // WE_LINE_NODE_H
