/**
 *	@file we_node_image_support.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 */

#ifndef WE_NODE_IMAGE_SUPPORT_H
#define WE_NODE_IMAGE_SUPPORT_H

#include <memory> //unique_ptr
#include <QSGTextureMaterial>

#include "../../../common/include/we_color.h"
#include "../../texture_cache/include/we_svg_texture_cache_key.h"
#include "../../image/include/we_gif_player.h"

#include "we_basic_geometry_node.h"

namespace WE
{

// Forward declarations:
class WEWidget;
class WEImageWidget;

class WEImage;
class ToolkitWEImage;

class WENodeImageSupport
{

public:

	explicit WENodeImageSupport(WEBasicGeometryNode& p_node, WEWidget *p_widget=nullptr);
	WENodeImageSupport(const WENodeImageSupport & p_copy) = delete;
	WENodeImageSupport& operator=(const WENodeImageSupport & p_copy) = delete;

	virtual ~WENodeImageSupport();

	void setTexture(QSGTexture *p_texture);
	void setImage(WEImage *p_image, const QSize& p_svgAndGifSize, const WEColor& p_svgBGColor=WEColor::transparentColor);
	void updateGIF();

	QSGTextureMaterial * getMaterial() const
	{
		return this->material;
	}

	QSGTexture * getTexture() const
	{
		return this->material->texture();
	}

	WEImage * getImage() const
	{
		return this->image;
	}

	static bool isNewImage(WEImage *p_newImage, WEImage *p_oldImage, ToolkitWEImage *p_oldToolkitImage, QSGTexture *p_oldTexture);

private:

	WEBasicGeometryNode & node;
	WEWidget * widget;
	WEImageWidget * imageWidget;
	bool widgetCasted;
	QSGTextureMaterial * material;
	WEImage * image;
	std::shared_ptr<ToolkitWEImage> toolkitImage;

	// SVG related members
	QSGTexture * svgTexture;
	WESVGTextureCacheKey svgCacheKey;
	WEColor svgBGColor;

	// GIF related members
	std::unique_ptr<WEGIFPlayer> gifPlayer{};	//!< WEGIFPlayer instance
	int currentGIFFrame;		//!< The frame number of the currently visible texture
	bool gifLoop;			//!< Repeat the GIF when reaches the end

	// SVG and GIF related members
	QSize size;

	void initGIFPlayer();
	bool isNewImage(WEImage * p_newImage);
	QSGTexture * getGIFTexture();
	QSGTexture * getSVGTexture();
	QImage * renderSVG();
	void releaseImage();
};
} //End of namespace WE
#endif // WE_NODE_IMAGE_SUPPORT_H
