/**
 * 	@file we_brush.h
 *
 *	@author maneesh.arora@aricent.com
 *	@author Maneesh Arora

 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 *
 *	@brief This class defines the Brush which defines the fill pattern of the
 *	shapes defined by WEPainter.
 *
 *	@see we_painter.h
 *
 *	This class defines the Brush which defines the fill pattern of the
 *	shapes defined by WEPainter.
 */

#ifndef WE_BRUSH_H_
#define WE_BRUSH_H_

#include "../../include/we_datatypes_gui.h"

namespace WE
{
class WEColor;
class WEImage;

/**
 *	@enum WEBrushStyle
 *
 *	@brief Definitions of the fill patterns that can be used.
 */
enum WEBrushStyle
{
	NO_BRUSH,				//!<No brush i.e No Fill
	SOLID_PATTERN,			//!<Uniform Color can be used with this style
	BRUSH_PATTERN_DIAGONAL_CROSS,
	TEXTURE_PATTERN		//!<Infinite repetition of specified pattern
							//!<(only for Image)
};

/**
 *	@struct WEBrush
 *
 *	@author maneesh.arora@aricent.com
 *	@author Maneesh Arora
 *
 *	@brief The WEBrush defines the fill pattern of shapes drawn by a WEPainter
 *
 *	@namespace WE
 *
 *	A brush has a style and a color.The brush style defines the fill pattern and
 *	and the brush color specifies the color to be used for filling.
 */
struct WEBrush
{
	WEColor color{255, 255, 255, 255};			//!< The brush color defines the color of the fill pattern
	WEBrushStyle brushStyle{SOLID_PATTERN};	//!< The brush style defines the fill pattern
	WEImage * textureImage{nullptr};		//!< Image to create pattern filling - nullptr for no pattern
	WEPoint textureOrigin{0, 0};		//!< Origin of the first pattern image in screen coordinates (for ISO-VT the top left corner of the data mask)

	WEBrush() = default;

	explicit WEBrush(WEBrushStyle p_style)
	:brushStyle(p_style)
	{
		//other members are initialized at declaration
	}

	virtual ~WEBrush()
	{
		// make sure that the one who set the texture image will care for memory management
		textureImage = nullptr;
	}

	/**
	 * Default copy constructor. Can be used because textureImage dynamic memory is managed by someone else
	 */
	WEBrush(const WEBrush & ) = default;

	/**
	 * Default assignment operator. Can be used because textureImage dynamic memory is managed by someone else
	 */
	WEBrush & operator=(const WEBrush &) = default;
};
}

#endif /* WEBRUSH_H_ */
