/**
 *	@file we_arched_bg_texture_cache_key.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 */

#ifndef WE_ARCHED_BG_CACHE_H
#define WE_ARCHED_BG_CACHE_H

#include <QSize>

#include "../../../common/include/we_color.h"

namespace WE
{

struct WEArchedBGTextureCacheKey
{
	WEWidgetTextureCacheKey widgetKey{};

	int absMinValue{-1};	//!< absolute min allowed for value
	int absMaxValue{-1};	//!< absolute max allowed for value
	int minValue{-1};		//!< change color if value is below this
	int maxValue{-1};		//!< change color if value is above this

	int graphWidth{-1};			//!< should be <= width / 2
	bool drawEnclosingBorder{false};	//!< Draw Enclosing Border true/false
	WEColor enclosingBorderColor{};	//!< color of the Enclosing Border


	bool operator<(const WEArchedBGTextureCacheKey& p_o) const
	{
		if (absMinValue != p_o.absMinValue)
		{
			return absMinValue < p_o.absMinValue;
		}
		else if (absMaxValue != p_o.absMaxValue)
		{
			return absMaxValue < p_o.absMaxValue;
		}
		else if (minValue != p_o.minValue)
		{
			return minValue < p_o.minValue;
		}
		else if (maxValue != p_o.maxValue)
		{
			return maxValue < p_o.maxValue;
		}
		else if (graphWidth != p_o.graphWidth)
		{
			return graphWidth < p_o.graphWidth;
		}
		else if (drawEnclosingBorder != p_o.drawEnclosingBorder)
		{
			return drawEnclosingBorder < p_o.drawEnclosingBorder;
		}
		else if (enclosingBorderColor != p_o.enclosingBorderColor)
		{
			return enclosingBorderColor < p_o.enclosingBorderColor;
		}

		return widgetKey < p_o.widgetKey;
	}
};
} //End of namespace WE
#endif // WE_ARCHED_BG_CACHE_H
