/**
 *	@file we_gauge_texture_cache_key.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 */

#ifndef WE_GAUGE_CACHE_H
#define WE_GAUGE_CACHE_H

#include "we_meter_texture_cache_key.h"

namespace WE
{

struct WEGaugeTextureCacheKey
{
	WEMeterTextureCacheKey meterKey{};
	WEColor colorBetween{};
	bool drawMinorTicks{true};
	WEColor minorTickColor{};
	unsigned int numberOfMinorTicks{0};
	double minorTickHeightFactor{1.0};
	double minorTickWidthFactor{1.0};
	bool drawValues{false};
	bool drawDigitalValue{false};
	double valuesDistanceFactor{1.0};
	bool fontAutoScaling{false};
	double fontScalingFactor{1.0};
	bool drawBoundaryBar{false};
	double boundaryBarWidthFactor{1.0};

	bool operator<(const WEGaugeTextureCacheKey& p_o) const
	{
		if (colorBetween != p_o.colorBetween)
		{
			return colorBetween < p_o.colorBetween;
		}
		else if (drawMinorTicks != p_o.drawMinorTicks)
		{
			return drawMinorTicks < p_o.drawMinorTicks;
		}
		else if (minorTickColor != p_o.minorTickColor)
		{
			return minorTickColor < p_o.minorTickColor;
		}
		else if (numberOfMinorTicks != p_o.numberOfMinorTicks)
		{
			return numberOfMinorTicks < p_o.numberOfMinorTicks;
		}
		else if (minorTickHeightFactor != p_o.minorTickHeightFactor)
		{
			return minorTickHeightFactor < p_o.minorTickHeightFactor;
		}
		else if (minorTickWidthFactor != p_o.minorTickWidthFactor)
		{
			return minorTickWidthFactor < p_o.minorTickWidthFactor;
		}
		else if (drawValues != p_o.drawValues)
		{
			return drawValues < p_o.drawValues;
		}
		else if (drawDigitalValue != p_o.drawDigitalValue)
		{
			return drawDigitalValue < p_o.drawDigitalValue;
		}
		else if (valuesDistanceFactor != p_o.valuesDistanceFactor)
		{
			return valuesDistanceFactor < p_o.valuesDistanceFactor;
		}
		else if (fontAutoScaling != p_o.fontAutoScaling)
		{
			return fontAutoScaling < p_o.fontAutoScaling;
		}
		else if (fontScalingFactor != p_o.fontScalingFactor)
		{
			return fontScalingFactor < p_o.fontScalingFactor;
		}
		else if (drawBoundaryBar != p_o.drawBoundaryBar)
		{
			return drawBoundaryBar < p_o.drawBoundaryBar;
		}
		else if (boundaryBarWidthFactor != p_o.boundaryBarWidthFactor)
		{
			return boundaryBarWidthFactor < p_o.boundaryBarWidthFactor;
		}

		return meterKey < p_o.meterKey;
	}
};
} //End of namespace WE
#endif // WE_GAUGE_CACHE_H
