/**
 *	@file we_boolean_widget.h
 *
 *	@author satheeskumar.p@aricent.com
 *	@author Satheeskumar Palanivel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 *
 *	@brief This files defines a boolean widget
 *
 *	@see WEWidget
 */
#ifndef WE_BOOLEAN_WIDGET_H_
#define WE_BOOLEAN_WIDGET_H_

//Included for data Members and Inherited classes
#include "we_button_widget.h"
#include "we_text_widget.h"

namespace WE
{
/**
 *	@class WEBooleanWidget
 *
 *	@author satheeskumar.p@aricent.com
 *	@author Satheeskumar Palanivel
 *
 *	@brief This class defines a BooleanWidget . A boolean widget is like a latchable button which can hold two
 *	values i.e. true/false . It displays a tick mark when it holds true value and false when it holds false.
 *	This class is inherited from WEButtonWidget because it has all the features of a button except that it can not
 *	hold values.
 *
 *	It also is a WETextWidget because the ISO 11783-6 input boolean object uses a font attributes object to define the color of the check mark.
 */
class WEBooleanWidget : public WEButtonWidget, public WETextWidget
{
	Q_OBJECT
public:

	/**
	 *	@fn WEBooleanWidget(WEWidget *p_parent, void * p_controller = nullptr, int p_typeID = WIDGET_TYPE_ID_BUTTON)
	 *
	 *	@brief Constructs a WEBooleanWidget.This widget can be created as
	 *	a separate window by specifying the parent as nullptr or it can be included
	 *	as a child widget of the parent.
	 *
	 *	@author satheeskumar.p@aricent.com
	 *	@author Satheeskumar Palanivel
	 *
	 *	@param[in] p_parent If parent IS nullptr then this widget becomes a window with the same properties.
	 *	@param[in] p_controller - pointer to controller - just to get it back via getController()
	 *	@param[in] p_typeID - macro to tell the type of widget to the constructor
	 */
	explicit WEBooleanWidget(WEWidget *p_parent, void * p_controller = nullptr, int p_typeID = WIDGET_TYPE_ID_BOOLEAN);

	/**
	 *	@fn ~WEBooleanWidget()
	 *
	 *	@brief Destructor of the widget.
	 *
	 *	@author satheeskumar.p@aricent.com
	 *	@author Satheeskumar Palanivel
	 */
	virtual ~WEBooleanWidget() = default;

	/**
	 *	@fn paint(WEPainter* p_painter)
	 *
	 *	@brief paints the boolean widget
	 *
	 *	@author satheeskumar.p@aricent.com
	 *	@author Satheeskumar Palanivel
	 *
	 *  @param[in] p_painter Pointer to painter the object
	 *
	 *  @returns void
	 */
	void paint(WEPainter* p_painter) override;

	/**
	 *	@fn init()
	 *
	 *	@brief the method to initialize the member variable of the class
	 *
	 *	@author satheeskumar.p@aricent.com
	 *	@author Satheeskumar Palanivel
	 *
	 *	 @returns void
	 */
	void init();
};	// class
}	// namespace

#endif /* WE_BOOLEAN_WIDGET_H_ */
