/*
 * we_graph_widget_types.h
 *
 *  Created on: May 15, 2019
 *      Author: dsl
 */

#ifndef SRC_GUI_WIDGETS_INCLUDE_WE_GRAPH_WIDGET_TYPES_H_
#define SRC_GUI_WIDGETS_INCLUDE_WE_GRAPH_WIDGET_TYPES_H_

#include <QPoint>

namespace WE
{

/**
 * For a documentation of the algorithms see psimpl.h in linux_3rd_party_libs
 */
typedef enum simplifyAlgorithmType
{
	SIMPLIFY_RADIAL_DISTANCE,       //!< SIMPLIFY_RADIAL_DISTANCE
	SIMPLIFY_PERPENDICULAR_DISTANCE,//!< SIMPLIFY_PERPENDICULAR_DISTANCE
	SIMPLIFY_REUMAN_WITKAM,         //!< SIMPLIFY_REUMAN_WITKAM
	SIMPLIFY_DOUGLAS_PEUCKER,       //!< SIMPLIFY_DOUGLAS_PEUCKER
	SIMPLIFY_OPHEIM,                //!< SIMPLIFY_OPHEIM
	SIMPLIFY_NO_SIMPLIFY            //!< SIMPLIFY_NO_SIMPLIFY
}simplifyAlgorithmType;

typedef enum XAxisLabelOptions
{
	FULL_X_AXIS_TIME_STAMPS 	= 0,
	RELATIVE_X_AXIS_TIME_STAMPS 	= 1,
	NO_X_AXIS_LABELS		= 2
}XAxisLabelOptions;

typedef enum YAxisPosition
{
	Y_AXIS_LEFT 	= 0,
	Y_AXIS_RIGHT	= 1,
	NO_Y_AXIS	= 2
}YAxisPosition;

typedef enum GridLineCount
{
	ONE_GRID_LINE 		= 1,
	THREE_GRID_LINES	= 3
}GridLineCount;

typedef enum LabelPosition
{
	LABEL_POSITION_TOP		= 0,
	LABEL_POSITION_THREE_FOURTH	= 1,
	LABEL_POSITION_MIDDLE		= 2,
	LABEL_POSITION_ONE_FOURTH	= 3,
	LABEL_POSITION_BOTTOM		= 4,
	LABEL_POSITION_INVALID		= 5
}LabelPosition;

/**
 *	@struct WEGraphData
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@brief Struct for saving a time stamp and the value at that time
 *
 *	@namespace WE
 *
 *	This struct is saved in a deque and represent the value history of a variable
 */
struct WEGraphData
{
	int value;	//!< value at time stamp
	unsigned long long timeStampInMilliseconds;
};

/**
 *	@struct WEVariableGraphOptions
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@brief Struct for saving drawing options for a variable in the graph
 *
 *	@namespace WE
 *
 *	This struct saves a lot of options that are necessary to draw the "value line" into the graph.
 */
struct WEVariableGraphOptions
{
	int 		minValue{0};	//!< the min value that is used to scale the y-axis (only if "!autoFitYAxis")
	int 		maxValue{0};	//!< the max value that is used to scale the y-axis (only if "!autoFitYAxis")
	double 		offset{0.0};		//!< if a value is displayed as label (at y axis or at cursor), this offset is used to calculate the displayed value
	double 		offset2{0.0};	//!< if a value is displayed as label (at y axis or at cursor), this offset2 is used to calculate the displayed value
	bool 		showYLabels{false};	//!< flag to tell if value labels shall be shown at y axis (e.g. min and max value)
	bool 		autoFitYAxis{false};	//!< flag to tell that the Y axis shall be automatically scaled based on the values currently visible
	unsigned short 	lineWidth{1};	//!< the width of the value line
	unsigned short 	linePattern{0};	//!< the value line can have a line pattern
	unsigned short 	numberOfDecimals{0};	//!< if a value is displayed as label (at y axis or at cursor), this number of decimals are shown
	double 		scale{1.0};		//!< if a value is displayed as label (at y axis or at cursor), this scale is used to calculate the displayed value
	WEColor 	lineColor{};	//!< the color for the value line

	QPoint		lastPointAddedToPolygon{}; //!< store the last point that was added to the polygon for this variable so we don't add the same point to the polygon twice
};
}  // namespace WE


#endif /* SRC_GUI_WIDGETS_INCLUDE_WE_GRAPH_WIDGET_TYPES_H_ */
