/**
 * 	@file we_qwidget_wrapper.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 *	@brief The WEQWidgetWrapper can be used to integrate QWidgets into the QtQuick scene
 *
 */

#ifndef WE_QWIDGET_WRAPPER_H_
#define WE_QWIDGET_WRAPPER_H_

#include <QPixmap>
#include <QPainter>
#include <QWidget>
#include <QQuickPaintedItem>

namespace WE
{

class WEQWidgetWrapper : public QQuickPaintedItem
{
	Q_OBJECT

public:

	/**
	 * @fn WEQWidgetWrapper(QQuickItem * parent, QWidget *widget=nullptr)
	 * @brief The WEQWidgetWrapper can be used to integrate QWidgets into the QtQuick scene
	 * @param[in] parent - pointer to parent widget
	 * @param[in] widget - pointer to the target widget (released in the destructor)
	 */
	explicit WEQWidgetWrapper(QQuickItem * p_parent, QWidget *p_widget=nullptr);

	virtual ~WEQWidgetWrapper();

	void setWidget(QWidget *p_widget);
	QWidget * getWidget();

	void paint(QPainter *p_painter) override;

	void mousePressEvent(QMouseEvent* p_qtMouseEvent) override;
	void mouseReleaseEvent(QMouseEvent* p_qtMouseEvent) override;


private:
	QPixmap pixmap;
	QWidget *widget;
};
} //End of namespace WE

#endif /* WE_QWIDGET_WRAPPER_H_ */
