/**
 *	@file qt_we_scrollbar.h
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2011
 *
 *	@brief See class documentation
 */


#ifndef QT_WE_SCROLLBAR_H_
#define QT_WE_SCROLLBAR_H_

#include <QScrollBar>

namespace WE {

/**
 *	@class QtWEScrollBar
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@brief Inherit from QScrollBar to be able to access mouse functions
 *
 *	@namespace WE
 *
 *	The only reason for this class is to be able to access the protected
 *	mouseReleaseEvent, mousePressEvent and mouseMoveEvent from QScrollBar.
 *	This is needed if a invisible container is located above the scrollbar
 *	so that the QWidget to which the scrollbar belongs can forward the mouse
 *	events to the scrollbar.
 */
class QtWEScrollBar : public QScrollBar
{
public:
	explicit QtWEScrollBar(QWidget * p_parent)
	:QScrollBar(p_parent)
	{
		this->setAttribute(Qt::WA_AcceptTouchEvents);
	}
	~QtWEScrollBar() = default;
	void mouseReleaseEvent ( QMouseEvent * p_event ) override
	{
		QScrollBar::mouseReleaseEvent(p_event);
	}
	void mousePressEvent ( QMouseEvent * p_event ) override
	{
		QScrollBar::mousePressEvent(p_event);
	}
	void mouseMoveEvent ( QMouseEvent * p_event ) override
	{
		QScrollBar::mouseMoveEvent(p_event);
	}
};
}  // namespace WE


#endif /* QT_WE_SCROLLBAR_H_ */
