/**
 *	@file qt_we_window_style.h
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2011
 *
 *	@brief See class documentation
 */


#ifndef QT_WE_WINDOW_STYLE_H_
#define QT_WE_WINDOW_STYLE_H_

#include <QStyle>
#include <QProxyStyle>

namespace WE
{


/**
 *	@class QtWEWindowStyle
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@brief Change QProxyStyle's pixelMetric method
 *
 *	@namespace WE
 *
 *	This class is only used to re-implement the pixelMetric function
 *	of the QStyle so that the up and down buttons of a scrollbar
 *	will get the correct height. Else they are too small to be used
 *	on a touch screen.
 */
class QtWEWindowStyle : public QProxyStyle
{
public:
	explicit QtWEWindowStyle(int p_height)
	:QProxyStyle(),
	 height(p_height)
	{
		//nothing more to do
	}
	~QtWEWindowStyle() = default;
private:
	int height;
	int pixelMetric(PixelMetric p_metric, const QStyleOption * /*p_option*/ = 0, const QWidget *p_pWidget = 0) const override
	{
		if ( p_metric == PM_ScrollBarExtent )
		{
			return height;
		}
		return  QProxyStyle::pixelMetric( p_metric, 0, p_pWidget );
	}
};
}	// namespace

#endif /* QT_WE_WINDOW_STYLE_H_ */
