/**
 *	@file we_can_message.h
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 *
 *	@brief Definition of a CAN message
 *
 *	This file defines the structure of a WE CAN message.
 */

#ifndef WE_CAN_MESSAGE_H_
#define WE_CAN_MESSAGE_H_


namespace WE
{
/**
 *	@struct WECANMessage
 *
 *	@brief Struct that defines a raw CAN message
 *
 */
struct WECANMessage
{
	WECANMessage() = default;

	WECANMessage(const WECANMessage &p_msg)
	:ID(p_msg.ID),
	 length(p_msg.length),
	 data{p_msg.data[0], p_msg.data[1], p_msg.data[2], p_msg.data[3], p_msg.data[4], p_msg.data[5], p_msg.data[6], p_msg.data[7]},
	 timeRx(p_msg.timeRx)
	{
		//nothing more to do here
	}

	virtual ~WECANMessage()
	{
		// need at least one virtual method to use dynamic cast identifying a derived class
		// and if the derived class allocates a virtual destructor is a good idea in any case.
	}

	WECANMessage & operator=(const WECANMessage &p_msg)
	{
		if (this == &p_msg)
		{
			return *this;
		}
		else
		{
			this->ID = p_msg.ID;
			this->length = p_msg.length;
			this->data[0] = p_msg.data[0];
			this->data[1] = p_msg.data[1];
			this->data[2] = p_msg.data[2];
			this->data[3] = p_msg.data[3];
			this->data[4] = p_msg.data[4];
			this->data[5] = p_msg.data[5];
			this->data[6] = p_msg.data[6];
			this->data[7] = p_msg.data[7];
			this->timeRx = p_msg.timeRx;
			return *this;
		}
	}

	void reset(void)
	{
		this->ID = 0;
		this->length = 0;
		this->data[0] = 0;
		this->data[1] = 0;
		this->data[2] = 0;
		this->data[3] = 0;
		this->data[4] = 0;
		this->data[5] = 0;
		this->data[6] = 0;
		this->data[7] = 0;
		this->timeRx = 0;
	}

	unsigned int ID{0};	//!< CAN Identifier
	unsigned int length{8};	//!< Number of data bytes of the CAN message
	mutable unsigned char data[8]{0, 0, 0, 0, 0, 0, 0, 0};	//!< Data bytes of the CAN message (mutable because inheriting classes want to return a non-const pointer to this array in const functions)
	unsigned int timeRx{0};	//!< Operating system time stamp when the can message was received by WECANRaw. WARNING: This time stamp can be newer than the one delivered by WETick!!! Never compare both of them without handling of this situation!!
};
}//End of namespace WE


#endif /* WE_CAN_MESSAGE_H_ */
