/**
 *	@file event_listener.h
 *
 *	@author dsterzel@topcon.com
 *	@author Dirk Sterzel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2016
 *
 *	@brief Definition of EventListener class
 *
 */

#ifndef SRC_HARDWARE_SUNRISE_HARDWARE_LIB_SRC_INCLUDE_EVENT_LISTENER_H_
#define SRC_HARDWARE_SUNRISE_HARDWARE_LIB_SRC_INCLUDE_EVENT_LISTENER_H_

#include <QObject>
#include "../../../../../event_handling/include/we_signal.h"

/**
 * This class implements the WESignalListener interface.
 * It's signalOccurred() method is called every time we receive a new or
 * changed hardware value which will in turn emit the variableValueChanged() signal.
 *
 */
class EventListener : public QObject, public WE::WESignalListener
{
	Q_OBJECT
public:
	/**
	 * Implementation of WESignalListener's abstract signalOccurred() function.
	 * It receives events from WEVariableEventDispatcher and emits the variableValueChanged() signal
	 * for every (int) value change we receive.
	 * @param p_signal A WESignal which in our case contains a (int) variable change event
	 * @return WE::WE_SUCCESS if signal was emitted, WE::WE_NOT_SUPPORTED_ERROR if event was not handled
	 */
	virtual int signalOccurred(WE::WESignal & p_signal);

	Q_SIGNALS:
	/**
	 * Signal that is emitted as soon as we receive a (int) variable value change event.
	 * @param p_key combined key of the variable (16Bit index and 16Bit subindex combined into one 32Bit value) => unique identifier
	 * @param p_value the new value of the variable identified by p_key
	 */
	void variableValueChanged(unsigned int p_key, int p_value);
};

#endif /* SRC_HARDWARE_SUNRISE_HARDWARE_LIB_SRC_INCLUDE_EVENT_LISTENER_H_ */
