/*
 * we_h264_stream.h
 *
 *  Created on: Aug 4, 2017
 *      Author: dsl
 */

#ifndef SRC_HARDWARE_VIDEO_INPUT_GSTREAMER_VIDEO_INPUT_INCLUDE_WE_GSTREAMER_VIDEO_INPUT_H_
#define SRC_HARDWARE_VIDEO_INPUT_GSTREAMER_VIDEO_INPUT_INCLUDE_WE_GSTREAMER_VIDEO_INPUT_H_

#include <map>
#include <QObject>

#include "../../../../os_abstraction/thread/include/we_thread.h"
#include "../../../../common/include/we_color.h"
#include "../../include/we_video_input_interface.h"

class QQuickItem;
class QQuickWidget;

typedef struct _GstElement GstElement;
typedef struct _GstMessage GstMessage;
namespace WE {

class WEThread;

enum VideoStandard
{
	STANDARD_PAL,
	STANDARD_NTSC,
	STANDARD_INVALID
};

struct VideoSetting
{
	WEVideoMirrorSetting mirrorSetting{WE::MIRRORED_NONE};
	WERotation rotation{0};
	int brightness{0};
	int contrast{127};
	int hue{0};
	int saturation{127};
	int standard{STANDARD_INVALID};
};

class WEGstreamerVideoInput : public QObject, public WEVideoInputInterface
{
	Q_OBJECT
public:
	static WEGstreamerVideoInput * getInstance(const std::string & p_analogVideoDevice);
	virtual ~WEGstreamerVideoInput();

	virtual int init(void) override;

	virtual WEVideoMirrorSetting getMirrorSetting(unsigned short p_cameraNumber) override;

	virtual WERotation getRotationSetting(unsigned short p_cameraNumber) override;


	virtual int getHeight(unsigned short p_cameraNumber) override;

	virtual int getWidth(unsigned short p_cameraNumber) override;

	virtual int getXPosition(unsigned short p_cameraNumber) override;

	virtual int getYPosition(unsigned short p_cameraNumber) override;

	virtual int setBrightness(unsigned short p_cameraNumber, int p_brightness) override;

	virtual int setContrast(unsigned short p_cameraNumber, int p_contrast) override;

	virtual int setHue(unsigned short p_cameraNumber, int p_hue) override;

	virtual int setMirror(unsigned short p_cameraNumber, WEVideoMirrorSetting p_mirrorSetting) override;

	virtual int setRotation(unsigned short p_cameraNumber, WERotation p_rotation) override;

	virtual int setPosition(unsigned short p_cameraNumber, int p_xPosition, int p_yPosition) override;

	virtual int setSaturation(unsigned short p_cameraNumber, int p_saturation) override;

	virtual int setSize(unsigned short p_cameraNumber, int p_width, int p_height) override;

	virtual int start(unsigned short p_cameraNumber) override;

	virtual int stop(unsigned short p_cameraNumber) override;

	virtual bool isRunning(unsigned short p_cameraNumber) override;

	virtual int setCurrentInterface(int p_cameraNumber) override;
	virtual int getCurrentInterface(void) const override;

	virtual bool isInitialized() override;

	unsigned int getTransparentColor(void) const;

	void setStreamType(int p_streamType);
	int getStreamType(void);

	void setRTSPURL(const std::string & p_url);
	const std::string & getRTSPURL(void);

	/**
	 * Set the QQuickItem where video shall be shown in (e.g. WEContainer).
	 * Do this before calling start()!
	 */
	void setVideoItem(QQuickItem * p_videoItem, WEColor p_backgroundColor);

	/**
	 * Call this function to remove the QQuickItem showing video (e.g. before that item is destroyed)
	 */
	void removeVideoItem(void);

	virtual void handleQueuedCommands() override;

	static void doQuit(void);


private:
	explicit WEGstreamerVideoInput(const std::string & p_analogVideoDevice);

	static void gstBusListener(void * p_instance);
	void createVideoAndWaitItem(void);

	/**
	 * static code analysis may report this function as unused but it is used in the functional implementation...
	 */
	void reset();

	GstElement * getPipeline(void);

	void setupCameraInput(int p_camera);
	void applyDetectedVideoStandard(int p_cam);
	void handleBusMessage(GstMessage * p_msg);

	static void doInitialSetup(void);

	static int detectVideoStandard(int p_cam);
	static void setVideoStandard(int p_camera, int p_std);

	static void waitForGstReady(void);

	static inline std::map<std::string, std::unique_ptr<WEGstreamerVideoInput>> instances{};

	int cameraID;
	int udpPort{0};
	bool initialized;
	unsigned int transparentColor;
	int x;
	int y;
	int width;
	int height;
	int streamType;
	std::string deviceFile;
	std::string rtspURL;
	QQuickItem * videoItem;
	QQuickItem * waitTextItem;
	QQuickItem * videoWidget;
	bool timedRestartStreamTriggered;
	bool running;
	unsigned int rtspStartTick;
	unsigned int restartCounter;
	static const unsigned int MAX_RESTARTS = 10;

	GstElement * pipeline;
	GstElement * sink;	//!< The video sink
	GstElement * glupload;
	GstElement * glcolorconvert; //needed on Linux PC only
	GstElement * src;	//!< uridecodebin element to automatically choose correct decoders for the given uri (e.g. file path)
	GstElement * filter;

	std::map<unsigned short, struct VideoSetting> videoSettings;

	static std::string videoDevices[3];
	static std::string videoSubDevices[3];
	static int fileDescriptors[3];
	static int fileDescriptorsSub[3];
	static int lastSetStandard[3];
	static std::string advName;
	static std::string twName;

	static bool runningOnQuadCore;

	static std::string palSize;
	static std::string ntscSize;

	static inline WEThread gstBusThread{};

	static inline bool quit{false};

	static inline bool gstInitDone{false};

private slots:
	void restartStream(void);
	void timedRestartStream(int p_timeout);
	void doShowVideoItem(bool p_show);
	void videoWidgetDestroyed(QObject * p_obj);
	int startSlot(void);
	void stopSlot(void);
	void changeInterfaceSlot(void);

signals:
	void errorDetected(int p_timeout);
	void triggerTimedRestart(int p_timeout);
	void showVideoItem(bool p_show);
};
}  // namespace WE



#endif /* SRC_HARDWARE_VIDEO_INPUT_GSTREAMER_VIDEO_INPUT_INCLUDE_WE_GSTREAMER_VIDEO_INPUT_H_ */
