/**
 *	@file we_environment_variable.h
 *
 *	@author hm@wachendorff.de
 *	@author Hendrik Mueller
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2012
 *
 *	@brief Interface for accessing envrionment variables
 */

#ifndef WE_ENVIRONMENT_VARIABLE_H_
#define WE_ENVIRONMENT_VARIABLE_H_

namespace WE
{

/**
 *	@class WEEnvironmentVariable
 *
 *	@author hm@wachendorff.de
 *	@author Hendrik Mueller
 *
 *	@brief The WEEnvironmentVariable class provides an interface for accessing environment variables
 *
 *	@namespace WE
 */
class WEEnvironmentVariable
{
public:
	/**
	 *	@fn static char* getEnvironmentVariable(const std::string &p_envVar)
	 *
	 *	@brief Returns the content of teh given environment variable
	 *
	 *	@author hm@wachendorff.de
	 *	@author Hendrik Mueller
	 *
	 *	@param p_envVar The environement variable to read
	 *
	 *	@returns the content of the given environment variable
	 *
	 *	This function deletes the instance.
	 */
	static char * getEnvironmentVariable(const std::string &p_envVar);

	/**
	 * Deleted copy constructor. If we need to copy this class please implement this.
	 */
	WEEnvironmentVariable(const WEEnvironmentVariable &) = delete;

	/**
	 * Deleted assignment operator. If we need to copy this class please implement this.
	 */
	WEEnvironmentVariable & operator=(const WEEnvironmentVariable &) = delete;

	// do not allow creating classes. It only contains static functions
	WEEnvironmentVariable() = delete;
	virtual ~WEEnvironmentVariable() = delete;
};
} // namespace WE

#endif /* WE_ENVIRONMENT_VARIABLE_H_ */
