/**
 *	@file we_network_linux.h
 *
 *	@author mka@wachendorff.de
 *	@author Markus Kalin
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 *
 *	@brief Interface for setting network configurations
 *
*/

#ifndef WE_NETWORK_LINUX_H_
#define WE_NETWORK_LINUX_H_

#include "../include/we_network.h"

namespace WE
{
	class WENetworkLinux : public WENetwork
	{

		// This class could benefit from some DOCUMENTATION, function headers, DOCUMENTATION, comments, DOCUMENTATION ... and documentation
		// Ask the author if you should not consider method names self-explanatory.

	public:
		WENetworkLinux() = default;
		virtual ~WENetworkLinux() = default;

		bool setNetworkConfig(WENetworkConfig* p_networkConfig) override;
		const WENetworkConfig* getNetworkConfig() const override;

		bool setNetworkSettings(const std::string & p_ipAddress, const std::string & p_netmask, const std::string & p_gateway) override;
		bool isIPAddressValid(const std::string & p_ipAddress) override;

		bool readNetworkSettings() override;
		bool applyNetworkSettings() override;
		void setNetworkinterface(const std::string & p_name) override;

		bool setIPAddress(const std::string & p_ipAddress, bool p_apply) override;
		bool setNetmaskIP(const std::string & p_netmask, bool p_apply) override;
		bool setGatewayIP(const std::string & p_gateway, bool p_apply) override;

		const std::string & getDefaultIPAdress() const override;
		const std::string & getDefaultNetmaskIP() const override;
		const std::string & getDefaultGatewayIP() const override;

	private:

		int setEthernet(const std::string & p_address, const std::string & p_mask);
		int setGateway(const std::string & p_gateway);
		int readGateway();
		static std::string executeCommand(const std::string & p_command);
		int readCurrentIpSettings();

		std::string networkinterface{"eth0"};

		std::string defaultIP{"192.168.135.6"};
		std::string defaultNetmask{"255.255.255.0"};
		std::string defaultGateway{"192.168.135.1"};

		WENetworkConfig networkConfig{defaultIP, defaultNetmask, defaultGateway};
		WENetworkConfig newNetworkConfig{defaultIP, defaultNetmask, defaultGateway};
	};
}//end of namespace WE

#endif /* WE_NETWORK_LINUX_H_ */
