/*
 * we_file_handling.h
 *
 *  Created on: 15.02.2011
 *      Author: mka
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2011
 */

#ifndef WE_FILE_HANDLING_H_
#define WE_FILE_HANDLING_H_

#include <condition_variable>
#include <mutex>

namespace WE
{

class WEFileHandlingInterface;
class WEThread;

class WEFileHandling
{

public:

	// This class could benefit from some DOCUMENTATION, function headers, DOCUMENTATION, comments, DOCUMENTATION ... and documentation
	// Ask the author if you should not consider method names self-explanatory.
	// This class seems to be exclusively used by project_variable_manager/project_variable_manager.cpp
	// mainly in order to copy log files while providing progress information

	WEFileHandling();
	virtual ~WEFileHandling();

	void release();

	void stop();

	static void copy(void* p_args);
	static void copyTheFile(void* p_args);

	std::unique_lock<std::mutex>* getMyLock() const
	{
		return this->myLock;
	}

	bool isStopped() const
	{
		return stopped;
	}

	bool isStarted() const
	{
		return started;
	}

	bool isCopy() const
	{
		return copyCommand;
	}

	void setThreadEnd(bool p_setEnd);
	int copyFile(const std::string & p_source, const std::string & p_target, bool p_copy, WEFileHandlingInterface* p_interface);
	int copyDir(const std::string & p_source, const std::string & p_target, bool p_copy, WEFileHandlingInterface* p_interface);

	const std::string & getSourceDirectory() const
	{
		return sourceDirectory;
	}

	const std::string & getTargetDirectory() const
	{
		return targetDirectory;
	}

	const std::string & getCurrentSourceFile() const
	{
		return currentSourceFile;
	}

	const std::string & getCurrentTargetFile() const
	{
		return currentTargetFile;
	}

	WEFileHandlingInterface* getInterface() const
	{
		return theInterface;
	}

	void join();

	int getError() const
	{
		return internalError;
	}

	void setError(int p_error)
	{
		internalError = p_error;
	}

	int getDebugSection() const;

private:

	bool started;
	bool stopped;

	bool ended;

	std::string sourceDirectory;
	std::string targetDirectory;
	std::string currentSourceFile;
	std::string currentTargetFile;

	int create(bool p_copy, WEFileHandlingInterface* p_interface);
	int setCurrentSourceFile(const std::string & p_file);
	int setCurrentTargetFile(const std::string & p_file);
	int setSourceDirectory(const std::string & p_directory);
	int setTargetDirectory(const std::string & p_directory);

	void start();

	void init();
	void reset();

	bool copyCommand;
	int internalError;

	WEFileHandlingInterface* theInterface;

	std::condition_variable condition;
	std::mutex the_mutex;
	std::unique_lock<std::mutex>* myLock;
	WEThread * myThread;
	int myDebugSection;
};
}
#endif /* WE_FILE_HANDLING_H_ */
