/**
 *	@file we_can_freestyle_resources.h
 *
 *	@author ju@wachendorff.de
 *	@author Joachim Urbach
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2012
 *
 *	@brief Implementation of CANFreestyle stack specific resources container.
 *
 */

#ifndef WE_CAN_FREESTYLE_RESOURCES_H_
#define WE_CAN_FREESTYLE_RESOURCES_H_


#include "../../common/include/we_can_protocol.h"


namespace WE
{


class WECANFreestyle;
class WEVariableEventDispatcher;
class WECANFreestyle;
class WEDEI;


/**
 *	@class WECANFreestyleResources
 *
 *	@brief Extends WECANProtocolResources with the CANFreestyle specific resources
 *
 *	This class contains all resources needed by the CANFreestyle stack. Using this class, it
 *	is easy to pass al important stuff to the statemachine too.
 *
 *	We expect them to be constant for a stack instance - so the members are also "const".
 *
 *	@see WECANProtocolResources
 */
class WECANFreestyleResources : public WECANProtocolResources
{

public:
	WECANFreestyle * const freestyleInstance;


	WECANFreestyleResources(WEDEI * p_dei,
			 WEVariableEventDispatcher * p_varEventDispatcher,
			 unsigned int p_portNo,
			 WECANFreestyle * p_freestyleInstance)
	: WECANProtocolResources(p_dei, p_varEventDispatcher, p_portNo),
	 freestyleInstance(p_freestyleInstance)
	{
		//this->freestyleInstance = p_freestyleInstance;
	}
	virtual ~WECANFreestyleResources()
	{
		//delete freestyleInstance;
	}
};
}
#endif /* WE_CAN_FREESTYLE_RESOURCES_H_ */
