/**
 *	@file we_can_ecu.h
 *
 *	@author hm@wachendorff.de
 *	@author Hendrik Mueller
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2011
 *
 *	@brief This file provides the basic information of an J1939 ECU.
 *
 */

#ifndef WE_J1939_CAN_ECU_H_
#define WE_J1939_CAN_ECU_H_

#include "../../common/include/we_can_ecu.h"
#include "we_j1939_transport_protocol_handler.h"
#include "we_j1939_definitions.h"

namespace WE
{

/**
 *	@struct WEJ1939CanEcu
 *
 *	@brief Struct that defines basic J1939 ECU properties.
 *
 */
class WEJ1939CanEcu : public WECanEcu
{
public:
	WEJ1939CanEcu() = default;

	virtual ~WEJ1939CanEcu() = default;

	//seems like we neither need to copy nor assign an ECU
	WEJ1939CanEcu(const WEJ1939CanEcu & p_ecu) = delete;
	WEJ1939CanEcu & operator=(const WEJ1939CanEcu &p_ecu) = delete;

	void reset(void)
	{
		WECanEcu::reset();
		this->address = J1939_NULL_ADDRESS;
		this->j1939Name = 0;
		this->j1939NameMask = 0;
		this->staticAddress = true;
	}

	unsigned long long j1939Name{0};		//!< The J1939 ECU name
	unsigned long long j1939NameMask{0};	//!< The J1939 ECU name mask - ecu is recognized if (ecuName & j1939NameMask) == (j1939Name & j1939NameMask)

	bool staticAddress{true};

	struct MessageAssembly sendMA{};	//!< Want to be able to send one TP to each ECU we know in parallel.

private:

	// SERIALIZATION
	// This class is exported in we_j1939_message_object.cpp, did not find any other suitable class/file in which to export.
	friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & p_ar, const unsigned int )
	{
		p_ar & boost::serialization::base_object<WECanEcu> (*this);	// identify parent class
		p_ar & j1939Name;		//!< The J1939 ECU name
		p_ar & j1939NameMask;	//!< The J1939 ECU name mask - ecu is recognized if (ecuName & j1939NameMask) == (j1939Name & j1939NameMask)
		// cppcheck-suppress bitwiseOnBoolean
		p_ar & staticAddress;
	}
};
}//End of namespace WE

#endif /* WE_J1939_CAN_ECU_H_ */
