/**
 *	@file Crc32.h
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2011
 *
 *	@brief Wrapper class for a crc32 algorithm
 *
 */


#ifndef CRC32_H_
#define CRC32_H_

#include <string>

namespace WE {

/**
 * This class is basically a wrapper around zlib's crc32 function.
  */
class WECRC32
{
public:
	/**
	 * Calculate crc32 checksum
	 * @param p_str	String containing the bytes for which the crc32 shall be calculated.
	 * @return The CRC32 checksum of p_str
	 */
	static unsigned int crc32(const std::string & p_str);

	WECRC32() = delete;
	~WECRC32() = delete;
};
}  // namespace WE


#endif /* CRC32_H_ */
