/**
 *	@file we_micro_time.h
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 */

#ifndef WE_MICRO_TIME_H_
#define WE_MICRO_TIME_H_

namespace WE
{

/**
 * 	struct systemTime
 *
 * 	@author hm@wachendorff.de
 * 	@author Hendrik Mueller
 *
 * 	@brief this structure is used to store the system time since the Epoch (1.1.1970)
 */
struct SystemTime
{
	unsigned int sec;	//!< seconds of system time since the Epoch (1.1.1970)
	unsigned int msec;	//!< milliseconds of system time since the Epoch (1.1.1970)
};

/**
 *	@class WEMicroTime
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@brief Class with static functions to retrieve high resolution times
 *
 *	@namespace WE
 */
class WEMicroTime
{
public:
	WEMicroTime() = delete;
	~WEMicroTime() = delete;

	/**
	 *	@brief SHORT DESCRIPTION
	 *
	 *	@author dsl@wachendorff.de
	 *	@author Dirk Sterzel
	 *
	 *	@returns The time in seconds since the Epoch
	 */
	static unsigned int getTimeInSeconds(void);

	/**
	 *	@fn static unsigned long long getTimeInMilliseconds(void)
	 *
	 *	@brief SHORT DESCRIPTION
	 *
	 *	@author dsl@wachendorff.de
	 *	@author Dirk Sterzel
	 *
	 *	@returns The time in milliseconds since the Epoch
	 */
	static unsigned long long getTimeInMilliseconds(void);

	/**
	 *	@fn static int getSystemTime(struct systemTime &time)
	 *
	 *	@brief Delivers the system time with precision of milliseconds.
	 *
	 *	@author hm@wachendorff.de
	 *	@author Hendrik Mueller
	 *
	 *	@param time The structure where to store the read time.
	 *
	 *	@returns WE_SUCCESS on success.
	 *	@returns WE_GENERAL_ERROR on any error. In case of error, time is not modified.
	 */
	static int getSystemTime(struct SystemTime &p_time);

	/**
	 *	@fn static int getUptime(struct systemTime &time)
	 *
	 *	@brief Delivers a monotonic system time with precision of milliseconds.
	 *
	 *	@author hm@wachendorff.de
	 *	@author Hendrik Mueller
	 *
	 *	@param time The structure where to store the read time.
	 *
	 *	@returns WE_SUCCESS on success.
	 *	@returns WE_GENERAL_ERROR on any error. In case of error, time is not modified.
	 */
	static int getUptime(struct SystemTime &p_time);
};
} // namespace WE

#endif /* WE_MICRO_TIME_H_ */
