/**
 *	@file we_size.h
 *
 *	@author maneesh.arora@aricent.com
 *	@author Maneesh Arora
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 *
 *	@brief This file defines the WESize class that contains the width and height.
 *
 */

#ifndef WE_SIZE_H_
#define WE_SIZE_H_

namespace WE
{

/**
 *	@class WESize
 *
 *	@author maneesh.arora@aricent.com
 *	@author Maneesh Arora
 *
 *	@brief Class that contains Size information i.e width and height
 *
 *	@namespace WE
 *
 *	This class contains size information(width and height).
 *
 */
class WESize
{
public:

	/**
	 *	@fn WESize() {}
	 *
	 *	@brief Default Constructor
	 *
	 *	@author maneesh.arora@aricent.com
	 *	@author Maneesh Arora
	 */
	WESize() = default;

	/**
	 *	@fn WESize(int width,int height) {}
	 *
	 *	@brief Constructor with width and height parameters
	 *
	 *	@author maneesh.arora@aricent.com
	 *	@author Maneesh Arora
	 *
	 *	@param[in] width int width of the size.
	 *	@param[in] height int height of the size.
	 *
	 */
	WESize(int p_width, int p_height);

	/**
	 *	@fn WESize(const WESize &size)
	 *
	 *	@brief Copy Constructor.
	 *
	 *	@author maneesh.arora@aricent.com
	 *	@author Maneesh Arora
	 *
	 *	@param[in] size WESize of which the copy is to be created.
	 */
	WESize(const WESize &p_size) = default;

	/**
	 *	@fn ~WESize() {}
	 *
	 *	@brief Destructor
	 *
	 *	@author maneesh.arora@aricent.com
	 *	@author Maneesh Arora
	 */
	virtual ~WESize() = default;

	/**
	 *	@fn getWidth()
	 *
	 *	@brief Returns the width.
	 *
	 *	@author maneesh.arora@aricent.com
	 *	@author Maneesh Arora
	 *
	 *	@returns int the width of the object.
	 */
	int getWidth() const;

	/**
	 *	@fn getHeight()
	 *
	 *	@brief Returns the height.
	 *
	 *	@author maneesh.arora@aricent.com
	 *	@author Maneesh Arora
	 *
	 *	@returns int the height of the object.
	 */
	int getHeight() const;

	/**
	 *	@fn setWidth(int width)
	 *
	 *	@brief Sets the width.
	 *
	 *	@author maneesh.arora@aricent.com
	 *	@author Maneesh Arora
	 *
	 *	@param[in] The width to be set.
	 */
	void setWidth(int p_width);

	/**
	 *	@fn setHeight(int height)
	 *
	 *	@brief Sets the height.
	 *
	 *	@author maneesh.arora@aricent.com
	 *	@author Maneesh Arora
	 *
	 *	@param[in] The height to be set.
	 */
	void setHeight(int p_height);

	/**
	 *	@fn operator=(const WESize &size)
	 *
	 *	@brief Equal to (=) Operator for the WESize
	 *
	 *	@author maneesh.arora@aricent.com
	 *	@author Maneesh Arora
	 *
	 *	@param[in] RHS of the = operator to be assigned to the LHS.
	 *
	 *	@returns WESize Reference to the LHS of the = operator.
	 */
	WESize& operator=(const WESize& p_size) = default;

	int width{0};	//!< width component
	int height{0};	//!< height component
}; // end of class WESize
}//End of Namespace WE
#endif /* WE_SIZE_H_ */
