/*
 * we_variable_definitions.h
 *
 *  Created on: May 29, 2019
 *      Author: dsl
 */

#ifndef SRC_DEI_VARIABLES_INCLUDE_WE_VARIABLE_DEFINITIONS_H_
#define SRC_DEI_VARIABLES_INCLUDE_WE_VARIABLE_DEFINITIONS_H_

namespace WE {

enum VARIABLE_DATA_TYPE_OFFSETS
{
	VARIABLE_DATA_TYPE_OFFSET_UNSIGNED = 0,
	VARIABLE_DATA_TYPE_OFFSET_SIGNED = 1000,
	VARIABLE_DATA_TYPE_OFFSET_STRING = 2000
};

/**
 *	@brief This enum defines the possible types of variables
 *
 *	@namespace WE
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 */
typedef enum {
	INTERNAL_VARIABLE, //!< Internal variable, includes hardware, project (PClient) and user defined variables
	EXTERNAL_VARIABLE,
	CANOPEN_VARIABLE, //!< CANOpen CAN protocol variable
	GENERICDRIVER_VARIABLE, //!< Generic Driver CAN protocol variable
	J1939_VARIABLE, //!< J1939 CAN protocol variable
	FREESTYLE_VARIABLE,
	VARIABLE_TYPE_UNDEFINED
//!< CAN freestyle variable
} WEVariableType;


/**
 *	@enum WEInternalVariableType
 *
 *	@brief Type definition for the different types of internal variables
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 */
typedef enum
{
	INTERNAL_VARIABLE_HARDWARE,	//!< hardware variable, index 0x2000 to 0x212f
	INTERNAL_VARIABLE_PROJECT,	//!< project variable (used by PClient), index 0x2130 to 0x2200
	INTERNAL_VARIABLE_UT_PLUGIN,	//!< used to control (and owned by) UT-Plugin (index 0x2500 ...)
	INTERNAL_VARIABLE_USER_DEFINED,	//!< user defined variable, index 0x3000 to 0x3fff
	INTERNAL_VARIABLE_UNDEFINED_TYPE
}WEInternalVariableType;

/**
 *	@brief A variable can represent a string or an integer value.
 *
 *	@namespace WE
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 */
typedef enum {
	STORAGE_TYPE_INTEGER = 0, //!< To be set if a variable represents integer values
	STORAGE_TYPE_STRING, //!< To be set if a variable represents string values
	STORAGE_TYPE_UNDEFINED
//!< initial value, for marking that something went terribly wrong
} WEVariableStorageType;

/**
 *	@brief With this type it can be determined if a variable is remanent, volatile or read only.
 *
 *	@namespace WE
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 */
typedef enum {
	ACCESS_TYPE_READ_WRITE,
	ACCESS_TYPE_WRITE_ONLY,
	ACCESS_TYPE_READ_ONLY,
	ACCESS_TYPE_CONSTANT,
	ACCESS_TYPE_UNDEFINED
} WEVariableAccessType;
}  // namespace WE



#endif /* SRC_DEI_VARIABLES_INCLUDE_WE_VARIABLE_DEFINITIONS_H_ */
