/*
 *	we_trigger.h
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 *	@brief Based on WEEvent and intended to communicate from hardware and data maintenance (fast thread) to operator input/drawing (slow thread).
 */

#ifndef SRC_EVENT_HANDLING_INCLUDE_WE_TRIGGER_H_
#define SRC_EVENT_HANDLING_INCLUDE_WE_TRIGGER_H_


#include <cstring>

#include "we_event.h"

namespace WE
{
// CLEAN_UP_EVENT
// WEEvent::object: pointer DDO to be cleaned;
// WEEvent::pointer: std::set<AppBaseDDO *> * generatedDDOs - list of DDOs to be deleted


// PROPERTY_CHANGE_EVENT
//struct propertyChangeParameter
//{
//	// use WEEvent::value: std::int32_t value;		//!< value - should be propertyID
//	// use WEEvent::object: void *object;		//!< internal used
//};

// SET_PROPERTY_IN_ALL_WIDGETS_EVENT
//struct setPropertyInAllWidgetsParameter
//{
//	// use WEEvent::value: std::int32_t propertyID;	//!< id of the property
//	// use WEEvent::object: void *object;		//!< internal used
//};

// SELECT_INPUT_OBJECT_EVENT
//struct selectInputObjectParameter
//{
//	// use WEEvent::uvalue: bool toBeOpenedForInput;	//!< true if 'open for input'true if 'open for input' is the desired state
//	// use WEEvent::controller: void *controller;		//!< reference to the controller belonging to the event
//	// use WEEvent::object: void *object;			//!< reference to the object belonging to the event
//	// use WEEvent::pointer: void *pool;			//!< reference to the pool belonging to the event
//};

// ESCAPE_INPUT_EVENT
// ESC triggered by command
//struct escapeInputParameter
//{
//	// use WEEvent::pointer: void *pool;	//!< the pool
//};

// REFRESH_AUX_SCREEN_EVENT
//struct auxPoolParameter
//{
//	// use WEEvent::pointer: void *pool;
//	// use WEEvent::object: void *keepTempAssignmentsFrom;
//};

// DELETE_OBJECT_POOL_EVENT
//struct deleteObjectPoolParameter
//{
//	// use WEEvent::pointer: void *poolToDelete;	//!< the pool to delete
//	// use WEEvent::object: void *poolToActivate;	//!< the pool to make active
//};

// TRIGGER_ALARM
//struct triggerAlarmParameter
//{
//	// use WEEvent::pointer: void * deadPool;
//	// use WEEvent::object: void * deadAuxInput;
//	// use WEEvent::uvalue: bool isAuxInputUnitDead;
//};

// CHANGE_ACTIVE_DATAMASK_EVENT
// this also exists as WEIncident to trigger macro
//struct changeActiveDatamaskParameter
//{
//	// use WEEvent::object: void *currentDatamask;	//!< current datamask
//	// use WEEvent::widget: void *newDatamask;	//!< the new datamask to be changed to
//	// use WEEvent::pointer: void *pool;		//!< reference to the pool belonging to the event
//};

// CHANGE_SOFTKEY_MASK_EVENT
// this also exists as WEIncident to trigger macro
//struct changeSoftkeyMaskParameter
//{
//	// use WEEvent::controller: void *currentSoftkeyMask;	//!< current softkey mask
//	// use WEEvent::pointer: void *newSoftkeyMask;		//!< new softkey mask
//	// use WEEvent::object void *object;			//!< the object referenced with the event
//};

// ADD_OBJECT_POOL_EVENT
// trigger redraw
//struct poolAddedParameter
//{
//	// use WEEvent::object: void * poolToAdd;	//!< the new pool to be added
//	// use WEEvent::pointer: void * poolToReplace;	//!< an old that should be replaced with the new pool
//};






union triggerParameter
{
};



struct WETrigger:virtual WEEvent
{
	// WEEvent.value is used for buttonValueToBeSet;	//!< Button needs a means to transport button value to other thread with the same event

	union triggerParameter para;

	/**
	 *	@brief Constructor directly defining the event type - UNKNOWN_EVENT if none provided
	 *
	 *	@param p_type The type of event to be created
	 *	@param p_class Keep default - is only modified by classes inheriting this one
	 *
	 *	@warning Logically p_type should clearly define what event class to generate but nothing is enforced there.
	 *
	 *	Marked as 'explicit' i.e. not allowed for implicit casting.
	 */
	explicit WETrigger(WEEventType p_type = UNKNOWN_EVENT, WEEventClass p_class = WE_TRIGGER)
	:WEEvent(p_type, p_class)
	{
		std::memset(&(this->para), 0, sizeof(this->para));
	}

	virtual ~WETrigger()
	{
		// nothing to clean up
	}


	inline void setPropertyChangeParameter(std::int32_t p_value, void * const p_object)
	{	// p_value sometimes is a value and sometimes is the property ID - not very consistent ... and somewhat duplicate with setSetPropertyParameter below?!
		this->value = p_value;				//!< value - should be propertyID
		this->object = p_object;			//!< internal used
	}

// totally unused it seems ... probably because setPropertyChangeParameter is used even where it shouldn't
//	inline void setSetPropertyParameter(std::int32_t p_propertyID, void * const p_object)
//	{
//		eventParameter.setProperty.propertyID = p_propertyID;
//		eventParameter.setProperty.object = p_object;
//	}

	inline void setSetPropertyInAllWidgetsParameter(std::int32_t p_propertyID, void * const p_object, std::uint32_t p_updateNavigation = 0)
	{
		this->value  = p_propertyID;		//!< id of the property
		this->object = p_object;		//!< internal used
		this->uvalue = p_updateNavigation;	//!< Update navigation after property applied to all widgets
	}

	inline void setGetCurrentVariableValueParameter(void * const p_object)
	{
		this->object = p_object;	//!< pointer to the object
	}

	inline void setChangeSoftkeyMaskParameter(void * const p_currentSoftkeyMask,
						  void * const p_newSoftkeyMask,
						  void * const p_object)
	{
		this->controller	= p_currentSoftkeyMask;		//!< current softkey mask
		this->pointer		= p_newSoftkeyMask;		//!< new softkey mask
		this->object		= p_object;			//!< the object referenced with the event
	}

	inline void setSelectInputObjectParameter(bool p_toBeOpenedForInput,
						  void * const p_controller,
						  void * const p_object,
						  void * const p_pool)
	{
		this->uvalue		= p_toBeOpenedForInput;	//!< true if 'open for input' is the desired state
		this->controller	= p_controller;		//!< reference to the controller belonging to the event
		this->object		= p_object;		//!< reference to the object belonging to the event
		this->pointer		= p_pool;		//!< reference to the pool belonging to the event
	}

	inline void setEscapeInputParameter(void * const p_pool)
	{
		this->pointer = p_pool;	//!< the pool
	}

	inline void setDeleteObjectPoolParameter(void * const p_poolToDelete,
						 void * const p_poolToActivate)
	{
		this->pointer	= p_poolToDelete;	//!< the pool to delete
		this->object	= p_poolToActivate;	//!< the pool to make active
	}

	inline void setShowAuxDeadAlarmParameter(void * p_poolToDelete, void * p_auxInputDead , bool p_auxInputUnitDead)
	{
		this->pointer	= p_poolToDelete;
		this->object	= p_auxInputDead;
		this->uvalue	= p_auxInputUnitDead;
	}

	inline void setChangeActiveDatamaskParameter(void * const p_currentDatamask,
						     void * const p_newDatamask,
						     void * const p_pool)
	{
		this->object	= p_currentDatamask;	//!< current datamask
		this->widget	= p_newDatamask;	//!< the new datamask to be changed to
		this->pointer	= p_pool;		//!< reference to the pool belonging to the event
	}

	inline void setPoolAddedParameter(void * const p_poolToAdd, void * const p_poolToReplace)
	{
		this->object	= p_poolToAdd;		//!< the new pool to be added
		this->pointer	= p_poolToReplace;	//!< an old that should be replaced with the new pool
	}
};



/**
 *	@brief Event listener interface that needs to be implemented to get notified of trigger events.
 *
 *	Subscription to events has to be implemented separately but they are expected to be handled in
 *	'triggerOccurred()'.
 */
class WETriggerListener
{
public:
	/**
	 *	@brief Handle events in this method.
	 *
	 *	@param[in] p_trigger The event that has occurred.
	 *
	 *	@returns an error code - WE_SUCCESS in case the event was successfully handled.
	 *
	 *	p_trigger should potentially be completed in this call but it should be carefully considered
	 *	to e.g. alter the event type since the event may be forwarded to other entities
	 *	after this method terminated. They are probably expecting the original content then.
	 *	I.e. generally p_trigger should be copied before the existing content is modified.
	 *
	 *	It is suggested to implement final event handling in the GUI context (slow thread) in this
	 *	method.
	 *	If they need handling in data context (fast thread) that should have happened before
	 *	they were considered 'WETrigger' only.
	 */
	virtual int triggerOccurred(WETrigger & p_trigger) = 0;
	virtual ~WETriggerListener() = default;
};
}	// namespace WE


#endif /* SRC_EVENT_HANDLING_INCLUDE_WE_TRIGGER_H_ */
