/**
 *	@file toolkit_we_painter_path.h
 *
 *	@author Tanmoy Deb
 *	@author tanmoy.deb@aricent.com
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 */

#ifndef TOOLKIT_WE_PAINTER_PATH_H_
#define TOOLKIT_WE_PAINTER_PATH_H_

namespace WE
{

class WERect;
class WEFont;
struct WEPoint;

/**
 *	@class ToolkitWEPainterPath
 *
 *	@author TANMOY.DEB AT ARICENT.COM
 *	@author TANMOY DEB
 *
 *	@brief Abstract interface of the toolkit specific painter path
 *
 *	@namespace WE
 */
class ToolkitWEPainterPath
{
public:
	/**
	 *	@fn ~ToolkitWEPainterPath()
	 *	@brief Destructor from the abstract PainterPath class
	 *
	 *	@author TANMOY.DEB AT ARICENT.COM
	 *	@author TANMOY DEB
	 *
	 *	ToolkitWEPainterPath is a completely abstract class
	 *	Therefore it has no Constructor and only the virtual Destructor, which must be defined
	 */
	virtual ~ToolkitWEPainterPath() = default;

	/**
	 *	@fn virtual WERect boundingRect() = 0
	 *
	 *	@brief Returns the bounding rectangle of this path.
	 *
	 *	@author TANMOY.DEB AT ARICENT.COM
	 *	@author TANMOY DEB
	 *
	 *	@returns The bounding rectangle of this path.
	 *
	 *	@see WERect.cpp
	 *
	 *	Returns the bounding rectangle of this path
	*/
	virtual WERect boundingRect() = 0;
	/**
	 *	@fn virtual ToolkitWEPainterPath* subtracted ( ToolkitWEPainterPath* p_toolkitWEPainterPath ) = 0
	 *
	 *	@brief Returns a PainterPath which is subtracted from this PainterPath.
	 *
	 *	@author tanmoy.deb AT aricent.com
	 *	@author Tanmoy Deb
	 *
	 *	@param[in] p_toolkitWEPainterPath The PainterPath to be subtracted from the another PainterPath.
	 *
	 *	@returns The result PainterPath
	 *
	*/
	virtual ToolkitWEPainterPath* subtracted ( ToolkitWEPainterPath* p_toolkitWEPainterPath ) = 0;

	/**
	 *	@fn virtual ToolkitWEPainterPath* united ( ToolkitWEPainterPath* p_toolkitWEPainterPath ) = 0
	 *
	 *	@brief Returns a PainterPath which is added with this PainterPath.
	 *
	 *	@author Tanmoy Deb
	 *	@author tanmoy.deb AT aricent.com
	 *
	 *
	 *	@param[in] p_toolkitWEPainterPath
	 *
	 *	@returns ToolkitWEPainterPath The result PainterPath after doing the united operation.
	 *
	 *
	*/
	virtual ToolkitWEPainterPath* united ( ToolkitWEPainterPath* p_toolkitWEPainterPath ) = 0;

	/**
	 *	@fn virtual int getStartAngle() = 0
	 *
	 *	@brief Return the start angle of a painter path
	 *
	 *	@author Tanmoy Deb
	 *	@author tanmoy.deb AT aricent.com
	 *
	 *	@returns Start angle of this painter path
	 */
	virtual int getStartAngle() const = 0;

	/**
	 *	@fn virtual int getSpanAngle() = 0
	 *
	 *	@brief Return the start angle of a painter path
	 *
	 *	@author Tanmoy Deb
	 *	@author tanmoy.deb AT aricent.com
	 *
	 *	@returns Span angle of this painter path
	 */
	virtual int getSpanAngle() const = 0;

	virtual void addText(WEFont &p_font, const std::string & p_text, int p_flag) = 0;

	virtual void addEllipse (WERect & p_rect) = 0;
	virtual void addPolygon (WEPoint * p_points, unsigned int p_numberOfPoints) = 0;

	virtual void setFillRule(int p_rule) = 0;
};
}
#endif
