/**
 *	@file qt_we_region.h
 *
 *	@author tanmoy deb AT aricent.com
 *	@author Tanmoy Deb
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 *
 *	@brief This file contains the definition of the QT specific definition of the WERegion
 *
 */

#ifndef QT_WE_REGION_H_
#define QT_WE_REGION_H_

#include <QtGui/QRegion>
#include "../../common/include/toolkit_we_region.h"
#include "../../common/include/we_region.h"

namespace WE
{
class WERect;

   /*
	*	This class inherit from the two basic classes:
	*	ToolkitWERegion - interface class for the Regions
	*	QRegion - graphic Region class
	*/
class QTWERegion : public ToolkitWERegion, public QRegion
{

public:
	/*--------------------------------------------------------*/
		/**
		 *	@fn QTWERegion(ToolkitWERegion *p_toolkitWERegion)
		 *
		 *	@brief Standard constructor of QTWERegion class
		 *
		 *	@author TANMOY.DEB AT ARICENT.COM
		 *	@author TANMOY DEB
		 *
		 *	@param p_toolkitWERegion Reference to the ToolkitWERegion for which this QTWERegion is created.
		 *
		 *	The constructor create the qt region interface.By default
		 *	the parent is nullptr indicating a window will be created.
		 */
		/*--------------------------------------------------------*/
		explicit QTWERegion(ToolkitWERegion *p_toolkitWERegion);

		/**
		 *	@fn  QTWERegion(WERect &p_rect,WE::WERegionType p_regionType)
		 *
		 *	@brief The standard constructor for the WERegion.
		 *
		 *	@author TANMOY.DEB AT ARICENT.COM
		 *	@author TANMOY DEB
		 *
		 *	@param[in] p_rect The rectangle of the region.
		 *	@param[in] p_regionType  This is the Rectangular or Elliptical Region.
		 *
		 *	The base Constructor of the QTWERegion.
		 */
		QTWERegion(WERect &p_rect,WE::WERegionType p_regionType);

		// default constructor
		QTWERegion();

		/*--------------------------------------------------------*/
		/**
		 *	@fn ~QTWERegion()
		 *
		 *	@brief Standard destructor of QTWERegion class
		 *
		 *	@author TANMOY.DEB AT ARICENT.COM
		 *	@author TANMOY DEB
		 *
		 *	The destructor deletes the qt Region interface.
		 */
		/*--------------------------------------------------------*/
		virtual ~QTWERegion() = default;

		/**
		 *	@fn WERect boundingRect()
		 *
		 *	@brief Returns the bounding rectangle of this region.
		 *
		 *	@author TANMOY.DEB AT ARICENT.COM
		 *	@author TANMOY DEB
		 *
		 *	@returns The bounding rectangle of this region.
		 *
		 *	@see WERect.cpp
		 *
		 *	Returns the bounding rectangle of this region.
		 */
		WERect boundingRect() override;

		/**
		 *	@fn ToolkitWERegion* subtract ( ToolkitWERegion  *p_toolkitWERegion )
		 *
		 *	@brief Returns a region which is subtracted from this region.
		 *
		 *	@author tanmoy.deb AT aricent.com
		 *	@author Tanmoy Deb
		 *
		 *	@param[in] p_toolkitWERegion The region to be substracted from the another region.
		 *
		 *	@returns The result region
		 *
		 */
		ToolkitWERegion* subtract ( ToolkitWERegion  *p_toolkitWERegion ) override;

		/**
		 *	@fn ToolkitWERegion* united ( ToolkitWERegion *p_toolkitWERegion )
		 *
		 *	@brief Returns a region which is added with this region.
		 *
		 *	@author Tanmoy Deb
		 *	@author tanmoy.deb AT aricent.com
		 *
		 *
		 *	@param[in] p_toolkitWERegion
		 *	@param[out] p_toolkitWERegion
		 *
		 *	@returns ToolkitWERegion The result region.
		 *
		 *
		 */
		ToolkitWERegion* united ( ToolkitWERegion *p_toolkitWERegion ) override;

private:
	// The private constructor of the QTWERegion
	explicit QTWERegion (const QRegion &p_region);
};
}

#endif /* QT_WE_REGION_H_ */
