/**
 *	we_gif_player.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 */

#ifndef WE_GIF_PLAYER_H
#define WE_GIF_PLAYER_H

#include <QTimer>
#include <QElapsedTimer>
#include <QMovie>
#include <QQuickItem>
#include <QSGTexture>
#include <QImageReader>

#include "../../texture_cache/include/we_gif_texture_cache_key.h"

namespace WE
{

/**
 *
 * This class can be used to play animated GIF images in the Qt Quick scene.
 * Use the currentTexture method to get the current image as QSGTexture.
 *
 */
class WEGIFPlayer : public QObject
{
	Q_OBJECT

public:
	WEGIFPlayer(const QString& p_fileName, const QSize& p_size, const QQuickItem *p_widget);
	virtual ~WEGIFPlayer();

	int currentFrameNumber() const;
	int frameCount() const;
	int speed() const;
	QMovie::MovieState state() const;
	QSGTexture * currentTexture();
	int nextFrameDelay(bool p_decodeIfNeeded=true);
	bool jumpToFrame(int p_frameNumber);

public slots:
	void start();
	void stop();
	void setPaused(bool p_paused);
	void setSpeed(int p_percentSpeed);
	void jumpToNextFrame();

signals:
	void started();
	void frameChanged(int p_frameNumber);
	void stateChanged(QMovie::MovieState p_state);

private:
	QImageReader *reader;
	QImage reusedImage;
	QTimer *timer;
	QElapsedTimer frameCreationElapsedTimer;
	int currentTimerInterval;
	int defaultFrameDelay;
	int totalFrameCount;
	int lastReadFrame;
	int previousFrame;
	int currentFrame;
	int currentSpeed;		// Default: 100%
	QMovie::MovieState currentState;
	int *frameDelay;		// Array with the duration for each frame
	QSGTexture **textureCache;	// Array with cached GIF textures
	bool noTextureCreated;
	bool globalTextureCacheUsed;
	WEGIFTextureCacheKey cacheKey;

	void setState(QMovie::MovieState p_state);
	bool allTexturesCreated();
	QSGTexture * getTexture(int p_frameNo);
	void callFrameChanged(int p_frameNumber);
	void restartTimer();
	void startTimer(int p_msec);
	void stopTimer();

private slots:
	void timerTimeout();
};
} //End of namespace WE

#endif // WE_GIF_PLAYER_H
