#ifndef WE_IMAGE_CONTAINER_H_
#define WE_IMAGE_CONTAINER_H_

/**
 *	@file we_image_container.h
 *
 *	@author mka@wachendorff.de
 *	@author Markus Kalin
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 *
 *	@brief Class definition of a container class for image creation and deletion
 *
 *  No get and set because the values are need directly and
 *  it is only a container.
 *
*/


namespace WE
{
class ToolkitWEImage;	//!< Forward declaration

/**
 *	@class WEImageContainer
 *
 *	@author mka@wachendorff.de
 *	@author Markus Kalin
 *
 *	@brief A container class that stores one image
 *
 *	@namespace WE
 */
class WEImageContainer
{
public:
	/**
	 *	@fn WEImageContainer()
	 *
	 *	@brief Default constructor
	 *
	 *	@author mka@wachendorff.de
 	 *	@author Markus Kalin
	 */
	WEImageContainer() = default;

	/**
	 *	@fn ~WEImageContainer()
	 *
	 *	@brief Default destructor
	 *
	 *	@author mka@wachendorff.de
 	 *	@author Markus Kalin
	 */
	virtual ~WEImageContainer() = default;

	// kept in map now: std::string strFilename;	//!< File name (path) of the image
	unsigned int uiInstances{0};	//!< Number of instances that use this image
	std::shared_ptr<ToolkitWEImage> poImage{nullptr};	//!< Pointer to the toolkit's representation of an image
};
}
#endif /*WE_IMAGE_CONTAINER_H_*/
