/**
 *	@file we_ellipse_node.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 */

#ifndef WE_ELLIPSE_NODE_H
#define WE_ELLIPSE_NODE_H

// Forward declarations:
namespace WE
{
class WEEllipseNode;
}

#include "we_basic_rectangle_node.h"
#include "we_geometry_node.h"

namespace WE
{

class WEControllerNode;

/**
 *
 *	This class can be used to draw an ellipse with the Qt quick scene graph.
 *	Note: This inherits 'WEBasicGeometryNode' from both ancestors.
 *
 */
class WEEllipseNode : public WEBasicRectangleNode, public WEGeometryNode
{

public:

	explicit WEEllipseNode(MaterialType p_materialType, const WERect& p_rect=WERect(0, 0, 100, 100));

	virtual ~WEEllipseNode() = default;

	void setFilled(bool p_filled);
	void setLineWidth(float p_lineWidth);
	void setRect(const WERect& p_r);
	void setSize(const QSize& p_s);
	void setCenter(const QPoint& p_center);
	void setRadius(int p_radius);
	void setRadius(int p_radiusX, int p_radiusY);
	void setRadiusX(int p_radiusX);
	void setRadiusY(int p_radiusY);
	void setStartAngle(int p_startAngle);
	void setEndAngle(int p_endAngle);
	void apply();

	bool getFilled() const
	{
		return this->filled;
	}

	float getLineWidth() const
	{
		return this->lineWidth;
	}

	QPoint getCenter() const
	{
		return this->center;
	}

	int getRadiusX() const
	{
		return getWidth();
	}

	int getRadiusY() const
	{
		return getHeight();
	}

	int getStartAngle() const
	{
		return startAngle;
	}

	int getEndAngle() const
	{
		return endAngle;
	}

	//Please node that setting p_isoAngles to true will not produce correct angles, this option should be avoided, see Jira PRJ-1034

	static void updateNode(WEEllipseNode * & p_node, WEControllerNode *p_controller, const WERect& p_rect, const WEColor& p_color, bool p_filled=true, float p_lineWidth=1, int p_startAngle=0, int p_endAngle=360, bool p_isoAngles=false);
	static void updateNode(WEEllipseNode * & p_node, WEControllerNode *p_controller, const QPoint& p_center, int p_radiusX, int p_radiusY, const WEColor& p_color, bool p_filled=true, float p_lineWidth=1, int p_startAngle=0, int p_endAngle=360, bool p_isoAngles=false);

	static void updateNode(WEEllipseNode * & p_node, WEControllerNode *p_controller, const WERect& p_rect, WEImage *p_image, bool p_filled=true, float p_lineWidth=1, int p_startAngle=0, int p_endAngle=360, bool p_isoAngles=false);
	static void updateNode(WEEllipseNode * & p_node, WEControllerNode *p_controller, const QPoint& p_center, int p_, int p_radiusY, WEImage *p_image, bool p_filled=true, float p_lineWidth=1, int p_startAngle=0, int p_endAngle=360, bool p_isoAngles=false);


protected:

	static void updateNode(WEEllipseNode * & p_node, WEControllerNode *p_controller, bool p_useRect, const WERect& p_rect, const QPoint& p_center, int p_radiusX, int p_radiusY, MaterialType p_materialType, const WEColor *p_color, WEImage *p_image, const QSize& p_geometrySize, bool p_filled=true, float p_lineWidth=1, int p_startAngle=0, int p_endAngle=360, bool p_isoAngles=false);

	void drawArc(float p_centerX, float p_centerY, float p_radiusX, float p_radiusY, unsigned int p_segments, unsigned int p_ellipseSegments, int p_startAngle = 0, int p_endAngle = 360, float p_lineWidth = 1);


	unsigned int segments{10};
	QPoint center{0, 0};
	int startAngle{0};
	int endAngle{0};
	bool filled{false};
	float lineWidth{1.0};
};
} //End of namespace WE
#endif // WE_ELLIPSE_NODE_H
