/**
 *	@file we_rectangle_node.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 */

#ifndef WE_RECTANGLE_NODE_H
#define WE_RECTANGLE_NODE_H

// Forward declarations:
namespace WE
{
class WEWidget;
class WERectangleNode;
class WEControllerNode;
}

#include "we_basic_rectangle_node.h"
#include "we_geometry_node.h"

#include "../../widgets/include/we_widget.h"
#include "../../common/include/we_rect.h"

namespace WE
{

/**
 *
 *	This class can be used to draw a rectangle with the Qt quick scene graph.
 *	Note: This inherits 'WEBasicGeometryNode' from both ancestors.
 *
 */
class WERectangleNode : public WEBasicRectangleNode, public WEGeometryNode
{

public:

	explicit WERectangleNode(MaterialType p_materialType);
	virtual ~WERectangleNode();

	void setFilled(bool p_filled);
	void setLineWidth(float p_lineWidth);
	void setCornerRadius(float p_cornerRadiusX, float p_cornerRadiusY);
	void apply();

	bool getFilled() const
	{
		return this->filled;
	}

	float getLineWidth() const
	{
		return this->lineWidth;
	}

	float getCornerRadiusX() const
	{
		return this->cornerRadiusX;
	}

	float getCornerRadiusY() const
	{
		return this->cornerRadiusY;
	}

	static void updateNode(WERectangleNode * & p_node, WEControllerNode *p_controller, const WERect& p_rect, const WEColor& p_color, bool p_filled=true, float p_lineWidth=1, float p_cornerRadiusX=0, float p_cornerRadiusY=0);
	static void updateNode(WERectangleNode * & p_node, WEControllerNode *p_controller, const WERect& p_rect, WEImage * p_image, bool p_filled=true, float p_lineWidth=1, float p_cornerRadiusX=0, float p_cornerRadiusY=0);


protected:

	void drawRoundedRect(QRectF p_rect, float p_radiusX, float p_radiusY, unsigned int p_segments, unsigned int p_ellipseSegments);

	static void updateNode(WERectangleNode * & p_node, WEControllerNode *p_controller, const WERect& p_rect, MaterialType p_materialType, const WEColor *p_color, WEImage * p_image, const QSize& p_geometrySize, bool p_filled=true, float p_lineWidth=1, float p_cornerRadiusX=0, float p_cornerRadiusY=0);

	bool filled;
	float lineWidth;
	float cornerRadiusX;
	float cornerRadiusY;
};
} //End of namespace WE
#endif // WE_RECTANGLE_NODE_H
