/**
 *	@file we_meter_texture_cache_key.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 */

#ifndef WE_METER_CACHE_H
#define WE_METER_CACHE_H

#include <QSize>

#include "../../include/we_datatypes_gui.h"
#include "../../../common/include/we_color.h"

namespace WE
{

struct WEMeterTextureCacheKey
{
	WEWidgetTextureCacheKey widgetKey{};

	int absMinValue{-1};	//!< absolute min allowed for value
	int absMaxValue{-1};	//!< absolute max allowed for value
	int minValue{-1};		//!< change color if value is below this
	int maxValue{-1};		//!< change color if value is above this

	WEColor colorBelowMin{};	//!< color to show if value is below min value
	WEColor colorNormal{};	//!< color to show if value is above min and below max value
	WEColor colorAboveMax{};	//!< color to show if value is above max value

	double tickAngleStep{1.0};	//!< saves the angle per tick
	double needleThicknessFactor{1.0};

	unsigned int middleDotThickness{1};

	WEDeflectionDirectionType direction{CLOCKWISE}; //!< stores current direction of the needle.

	bool drawTicks{false}; //!< Draw ticks true/false
	bool drawArc{false}; //!< Draw ticks true/false
	double arcWidthFactor{1.0};
	bool drawEnclosingBorder{false}; //!< Draw Enclosing Border true/false

	WEColor tickColor{}; //!< color of ticks
	WEColor arcColor{}; //!< color of the arc
	WEColor enclosingBorderColor{}; //!< color of the Enclosing Border

	std::uint16_t numberOfTicks{0}; //!< Number of ticks to be drawn

	double tickHeightFactor{1.0};
	double tickWidthFactor{1.0};
	bool drawTickPoint{false};
	double tickPointSizeFactor{1.0};
	WEColor tickPointColor{};
	WETickPointFormatType tickPointFormat{NO_FORMAT};


	bool operator<(const WEMeterTextureCacheKey& p_o) const
	{
		//value
		if (absMinValue != p_o.absMinValue)
		{
			return absMinValue < p_o.absMinValue;
		}
		else if (absMaxValue != p_o.absMaxValue)
		{
			return absMaxValue < p_o.absMaxValue;
		}
		else if (minValue != p_o.minValue)
		{
			return minValue < p_o.minValue;
		}
		else if (maxValue != p_o.maxValue)
		{
			return maxValue < p_o.maxValue;
		}
		//color
		else if (colorBelowMin != p_o.colorBelowMin)
		{
			return colorBelowMin < p_o.colorBelowMin;
		}
		else if (colorNormal != p_o.colorNormal)
		{
			return colorNormal < p_o.colorNormal;
		}
		else if (colorAboveMax != p_o.colorAboveMax)
		{
			return colorAboveMax < p_o.colorAboveMax;
		}
		//other
		else if (tickAngleStep != p_o.tickAngleStep)
		{
			return tickAngleStep < p_o.tickAngleStep;
		}
		else if (needleThicknessFactor != p_o.needleThicknessFactor)
		{
			return needleThicknessFactor < p_o.needleThicknessFactor;
		}
		else if (middleDotThickness != p_o.middleDotThickness)
		{
			return middleDotThickness < p_o.middleDotThickness;
		}
		else if (direction != p_o.direction)
		{
			return direction < p_o.direction;
		}
		else if (drawTicks != p_o.drawTicks)
		{
			return drawTicks < p_o.drawTicks;
		}
		else if (drawArc != p_o.drawArc)
		{
			return drawArc < p_o.drawArc;
		}
		else if (arcWidthFactor != p_o.arcWidthFactor)
		{
			return arcWidthFactor < p_o.arcWidthFactor;
		}
		else if (drawEnclosingBorder != p_o.drawEnclosingBorder)
		{
			return drawEnclosingBorder < p_o.drawEnclosingBorder;
		}
		else if (tickColor != p_o.tickColor)
		{
			return tickColor < p_o.tickColor;
		}
		else if (arcColor != p_o.arcColor)
		{
			return arcColor < p_o.arcColor;
		}
		else if (enclosingBorderColor != p_o.enclosingBorderColor)
		{
			return enclosingBorderColor < p_o.enclosingBorderColor;
		}
		else if (numberOfTicks != p_o.numberOfTicks)
		{
			return numberOfTicks < p_o.numberOfTicks;
		}
		else if (tickHeightFactor != p_o.tickHeightFactor)
		{
			return tickHeightFactor < p_o.tickHeightFactor;
		}
		else if (tickWidthFactor != p_o.tickWidthFactor)
		{
			return tickWidthFactor < p_o.tickWidthFactor;
		}
		else if (drawTickPoint != p_o.drawTickPoint)
		{
			return drawTickPoint < p_o.drawTickPoint;
		}
		else if (tickPointSizeFactor != p_o.tickPointSizeFactor)
		{
			return tickPointSizeFactor < p_o.tickPointSizeFactor;
		}
		else if (tickPointColor != p_o.tickPointColor)
		{
			return tickPointColor < p_o.tickPointColor;
		}
		else if (tickPointFormat != p_o.tickPointFormat)
		{
			return tickPointFormat < p_o.tickPointFormat;
		}

		return widgetKey < p_o.widgetKey;
	}
};
} //End of namespace WE
#endif // WE_METER_CACHE_H
