/**
 *	@file we_svg_texture_cache_key.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 */

#ifndef WE_SVG_CACHE_H
#define WE_SVG_CACHE_H

#include <string>
#include <QSize>

#include "../../../common/include/we_color.h"

namespace WE
{

struct WESVGTextureCacheKey
{
	std::string filePath;
	QSize size;
	WEColor bgColor;

	WESVGTextureCacheKey(const std::string& p_filePath="", const QSize& p_size=QSize(), const WEColor& p_bgColor=WEColor::transparentColor)
	:filePath(p_filePath), size(p_size), bgColor(p_bgColor)
	{
		//nothing more to do
	}

	bool operator<(const WESVGTextureCacheKey& p_o) const
	{
		if (size.width() != p_o.size.width())
		{
			return size.width() < p_o.size.width();
		}
		else if (size.height() != p_o.size.height())
		{
			return size.height() < p_o.size.height();
		}
		else if (bgColor != p_o.bgColor)
		{
			return bgColor < p_o.bgColor;
		}

		return filePath < p_o.filePath;
	}
};
} //End of namespace WE
#endif // WE_SVG_CACHE_H
