/**
 *	@file we_widget_texture_cache_key.h
 *
 *	@author preths@topcon.com
 *	@author Patrick Reths
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2020
 *
 */

#ifndef WE_WIDGET_CACHE_H
#define WE_WIDGET_CACHE_H

#include <QSize>

#include "../../include/we_datatypes_gui.h"
#include "../../common/include/we_border_profile.h"
#include "../../../common/include/we_color.h"

namespace WE
{

struct WEWidgetTextureCacheKey
{
	QSize size{};

	WEBorderProfile borderProfile{};	//!<Object with the border profile properties
	std::string backgroundImagePath{};
	std::string borderImagePath{};

	int borderWidth{0};		//!<Width of the Border of the Widget
	int startAngle{0};			//!<needed for the border profile
	int endAngle{0};			//!<needed for the border profile
	int spanAngle{0};
	bool drawBorderFlag{false};		//!<If the Border should be drawn
	int cornerRadius{0};
	bool transparentBorderFlag{false};	//!<Shold the Border be transparent
	bool drawBorderImageFlag{false};
	bool drawBackgroundImageFlag{false};
	bool usesForeignBorderProfile{false};	//!< MKA's undocumented magic ... creating segfaults if one just wants to set a border profile and chooses the method that promises to create a copy :-(
	// it seems true == usesForeignBorderProfile should indicate that we are sharing somebody else's border profile
	// whereas false == usesForeignBorderProfile means we are maintaining our own copy
	WEBorderState borderState{BORDER_STATE_UNDEFINED};	//!< Current State of the widget (DISABLED, Enabled with Focus, Enabled without Focus, Open for Input)

	WEColor transparentBorderColor{};
	WEColor foregroundColor{};	//!<Foreground Color of the Widget
	WEColor backgroundColor{};	//!<Background Color of the Widget


	bool operator<(const WEWidgetTextureCacheKey& p_o) const
	{
		if (size.width() != p_o.size.width())
		{
			return size.width() < p_o.size.width();
		}
		else if (size.height() != p_o.size.height())
		{
			return size.height() < p_o.size.height();
		}
		else if (borderProfile != p_o.borderProfile)
		{
			return borderProfile < p_o.borderProfile;
		}
		else if (backgroundImagePath != p_o.backgroundImagePath)
		{
			return backgroundImagePath < p_o.backgroundImagePath;
		}
		else if (borderImagePath != p_o.borderImagePath)
		{
			return borderImagePath < p_o.borderImagePath;
		}
		else if (borderWidth != p_o.borderWidth)
		{
			return borderWidth < p_o.borderWidth;
		}
		else if (startAngle != p_o.startAngle)
		{
			return startAngle < p_o.startAngle;
		}
		else if (endAngle != p_o.endAngle)
		{
			return endAngle < p_o.endAngle;
		}
		else if (spanAngle != p_o.spanAngle)
		{
			return spanAngle < p_o.spanAngle;
		}
		else if (drawBorderFlag != p_o.drawBorderFlag)
		{
			return drawBorderFlag < p_o.drawBorderFlag;
		}
		else if (cornerRadius != p_o.cornerRadius)
		{
			return cornerRadius < p_o.cornerRadius;
		}
		else if (transparentBorderFlag != p_o.transparentBorderFlag)
		{
			return transparentBorderFlag < p_o.transparentBorderFlag;
		}
		else if (drawBorderImageFlag != p_o.drawBorderImageFlag)
		{
			return drawBorderImageFlag < p_o.drawBorderImageFlag;
		}
		else if (drawBackgroundImageFlag != p_o.drawBackgroundImageFlag)
		{
			return drawBackgroundImageFlag < p_o.drawBackgroundImageFlag;
		}
		else if (usesForeignBorderProfile != p_o.usesForeignBorderProfile)
		{
			return usesForeignBorderProfile < p_o.usesForeignBorderProfile;
		}
		else if (borderState != p_o.borderState)
		{
			return borderState < p_o.borderState;
		}
		else if (transparentBorderColor != p_o.transparentBorderColor)
		{
			return transparentBorderColor < p_o.transparentBorderColor;
		}
		else if (foregroundColor != p_o.foregroundColor)
		{
			return foregroundColor < p_o.foregroundColor;
		}

		return backgroundColor < p_o.backgroundColor;
	}
};
} //End of namespace WE
#endif // WE_WIDGET_CACHE_H
