/**
 *	@file we_message_box.h
 *
 *	@author dsterzel@topcon.com
 *	@author Dirk Sterzel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2016
 *
 *	@brief Definition of WEMessageBox class. See class description for more info
 */

#ifndef SRC_GUI_WIDGETS_INCLUDE_WE_WARNING_MESSAGE_BOX_H_
#define SRC_GUI_WIDGETS_INCLUDE_WE_WARNING_MESSAGE_BOX_H_

#include <QMessageBox>

namespace WE {

/**
 * WEWarningMessageBox is a small wrapper around QMessageBox that
 * can be used to show a warning message. With the optional autoCloseTimer
 * member, the warning message can be configured to automatically close
 * after the defined milliseconds.
 * WEWarningMessageBox is used in WEWindowWidget::showModalWarningMessage()
 */
class WEWarningMessageBox : public QMessageBox
{
	Q_OBJECT
public:
	WEWarningMessageBox();
	~WEWarningMessageBox() = default;
	explicit WEWarningMessageBox(QWidget * p_parent);
	//starts the auto close timer if it was configured to be > 0
	void showEvent (QShowEvent * p_event ) override;
	//closes the dialog after timer elapsed
	void timerEvent(QTimerEvent * p_event) override;

	void setAutoCloseTimer(int p_timer);
protected:
	int autoCloseTimer;
};
}  // namespace WE



#endif /* SRC_GUI_WIDGETS_INCLUDE_WE_WARNING_MESSAGE_BOX_H_ */
