/**
 *	@file qt_we_scroll_area.h
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2011
 *
 *	@brief Definition of WTWEScrollArea class
 */


#ifndef QT_WE_SCROLL_AREA_H_
#define QT_WE_SCROLL_AREA_H_

#include <QScrollArea>
#include "../include/we_widget.h"

namespace WE {

/**
 *	@class QTWEScrollArea
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@brief Class inheriting QScrollArea only for key forwarding
 *
 *	@namespace WE
 *
 *	It seems like QScrollArea somehow grabs the complete keyboard focus and because
 *	of that no key events were sent to QTWEWidget anymore when a QScrollArea was used.
 *	This class inherits QScrollArea and implements the keyPressEvent and keyReleaseEvent
 *	functions. In these functions the key events are just forwarded to the repective
 *	functions of the QTWEWidget inside the scroll area.
 */
class QTWEScrollArea : public QScrollArea
{
public:
	explicit QTWEScrollArea(QWidget * parent = nullptr)
	:QScrollArea(parent)
	{
		//nothing more to do
	}
protected:
	/**
	 * See class description
	 */
	void keyPressEvent ( QKeyEvent * e ) override
	{
		WEWidget * w = dynamic_cast<WEWidget*>(this->widget());
		if (nullptr != w)
		{
			w->keyPressEvent(e);
		}
	}

	/**
	 * See class description
	 */
	void keyReleaseEvent ( QKeyEvent * e ) override
	{
		WEWidget * w = dynamic_cast<WEWidget*>(this->widget());
		if (nullptr != w)
		{
			w->keyReleaseEvent(e);
		}
	}
};
}  // namespace WE


#endif /* QT_WE_SCROLL_AREA_H_ */
