/**
 *	@file windows_can_interface_peak.h
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 *
 *	@brief Definition of an interface to access the XXX interface.
 *
 *	@see can_interface.h
 *	@see we_can_raw.h
 *
 *	The class defined in this file allows access to the XXX CAN interface
 */


#ifndef WINDOWS_CAN_INTERFACE_PEAK_H_
#define WINDOWS_CAN_INTERFACE_PEAK_H_


#include <vector>

#include <CanApi2.h>

#include "../include/we_can_raw.h"

namespace WE
{

class WEMicroTime;

class CANWindowsPeak : public WECANRaw
{
private:

	bool initialized;	//!< indicates if the interface has been correctly initialized
	bool pollingRead;	//!< indicates if polling is enabled or not
	unsigned int pollIntervall;	//!< time in milliseconds in which shall be polled

	int endPointID;
	bool connected;

	int numberOfRetries;
	int retryTimeout;

	HCANCLIENT clientHandle;
	TCANMsg * recvMsg;
	TCANMsg * sendMsg;
	HCANNET netHandle;

public:

	// This class could benefit from some DOCUMENTATION, function headers, DOCUMENTATION, comments, DOCUMENTATION ... and documentation
	// Ask the author if you should not consider method names self-explanatory.

	CANWindowsPeak();
	//don't allow copy or assignment
	CANWindowsPeak(const CANWindowsPeak & p_copy) = delete;
	CANWindowsPeak& operator=(const CANWindowsPeak& p_copy) = delete;


	explicit CANWindowsPeak(const char *  interfaceName);


	virtual ~CANWindowsPeak();

	int initialize(unsigned int portNo, unsigned int baudrateToSet, const char * interfaceName) override;

	int initialize(unsigned int p_portNo) override;

	int interfaceInit(const char *  interfaceName);

	int receive(WECANMessage * p_messageToStoreReceivedDataTo) override;

	int send(WECANMessage * messageToSend) override;

	void setPolling(bool polling, unsigned int timeout) override;

	virtual int getReadEndPointID() const override;

	virtual int getWriteEndPointID() const override;

	virtual bool isConnected() const override;

	virtual int reconnect() override;

	int addIDFilter(unsigned int ID, unsigned int mask) override;

};

}//end of namespace WE

#endif /* WINDOWS_CAN_INTERFACE_PEAK_H_ */
