/**
 *	@file we_eeprom.h
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2011
 *
 */

#ifndef WE_EEPROM_H_
#define WE_EEPROM_H_


#include <string>


namespace WE
{

	// This header could benefit from some DOCUMENTATION, function headers, DOCUMENTATION, comments, DOCUMENTATION ... and documentation
	// Ask the author if you should not consider names self-explanatory.

typedef enum
{
	EEPROM_CMD_GET_OTC = 0,
	EEPROM_CMD_OTC,
	EEPROM_CMD_GET_EE_PRODUCT_TYPE,
	EEPROM_CMD_EE_PRODUCT_TYPE,
	EEPROM_CMD_GET_EE_PRODUCT_VARIANT,
	EEPROM_CMD_EE_PRODUCT_VARIANT,
	EEPROM_CMD_GET_EE_BOARD_REVISION,
	EEPROM_CMD_EE_BOARD_REVISION,
	EEPROM_CMD_GET_EE_BOARD_NAME,
	EEPROM_CMD_EE_BOARD_NAME,
	EEPROM_CMD_GET_EE_PART_NUMBER,
	EEPROM_CMD_EE_PART_NUMBER,
	EEPROM_CMD_GET_EE_CUSTOMER_PART_NUMBER,
	EEPROM_CMD_EE_CUSTOMER_PART_NUMBER,
	EEPROM_CMD_GET_EE_SERIAL_NUMBER,
	EEPROM_CMD_EE_SERIAL_NUMBER,
	EEPROM_CMD_GET_EE_BOOT_MODE,
	EEPROM_CMD_SET_EE_BOOT_MODE,
	EEPROM_CMD_EE_BOOT_MODE,
	EEPROM_CMD_GET_EE_UNIQUE_RELEASE_ID,
	EEPROM_CMD_EE_UNIQUE_RELEASE_ID,
	EEPROM_CMD_GET_EE_DELETE_DIAG,
	EEPROM_CMD_EE_DELETE_DIAG,
	EEPROM_CMD_GET_EE_CONSOLE_MODE,
	EEPROM_CMD_EE_CONSOLE_MODE,
	EEPROM_CMD_GET_API_VERSION,
	EEPROM_CMD_API_VERSION,
	EEPROM_CMD_ERROR
} WEEEPROMCommand;

typedef enum
{
	WEHMD_ERR_UNKNOWN_COMMAND=0,
	WEHMD_ERR_EEPROM_NOT_AVAILABLE,
	WEHMD_ERR_COULD_NOT_READ,
	WEHMD_ERR_NOT_A_CLIENT_COMMAND
} WEEEPROMErrorType;

typedef struct
{
	/* command which caused the error */
	WEEEPROMCommand command;
	/* type of error */
	WEEEPROMErrorType errorType;
} WEEEPROMError;

typedef union
{
	unsigned int otc;
	unsigned char productType;
	unsigned char productVariant;
	unsigned char boardRevision;
	char boardName[31];
	char partNumber[33];	// A3/A6: 20 bytes; A5: 32 bytes
	char customerPartNumber[33];	// A3/A6 32 bytes A5: 20 bytes
	char serialNumber[20];	// A3/A6: 20 bytes; A5: 12 bytes
	unsigned char bootMode;
	unsigned int uniqueReleaseID;
	unsigned char deleteDiag;
	unsigned char consoleMode;
	unsigned int apiVersion;
	WEEEPROMError error;
} WEEEPROMMessageData;

class WEEeprom
{
public:
	WEEeprom() = delete;
	virtual ~WEEeprom() = default;

	static int readDataFromEeprom(WEEEPROMCommand, WEEEPROMMessageData & p_data);
	static int writeDataToEeprom(WEEEPROMCommand, WEEEPROMMessageData & p_data);

	static int getProductType();
	static int getProductVariant();
	static std::string getSerialNumber();
	static std::string getPartNumber();
	static std::string getCustomerPartNumber();

private:

	static void fillCommandMap(void);
	static int initEeprom();
	static int initEepromDev(std::string p_dev, char p_name[4], char p_id[7], std::string p_attributePath );

	static int sendCommandToHealthDaemon(void * p_msg);
	static int getEEPROMData(WEEEPROMCommand p_eepromCommand, WEEEPROMMessageData & p_data);
};
}


#endif /* WE_EEPROM_H_ */
