/**
 *	@file we_network.h
 *
 *	@author mka@wachendorff.de
 *	@author Markus Kalin
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 *
 *	@brief Interface for setting network configurations
 *
 */

#ifndef WE_NETWORK_H_
#define WE_NETWORK_H_

#include <string>

namespace WE
{
	struct WENetworkConfig
	{
		WENetworkConfig() = default;

		WENetworkConfig(const std::string & p_ip, const std::string & p_mask, const std::string & p_gw)
		:ipAddress(p_ip),
		netmask(p_mask),
		gateway(p_gw)
		{
			//nothing more to do here
		}

		std::string ipAddress{};
		std::string netmask{};
		std::string gateway{};
	};

	class WENetwork
	{
	public:
		virtual ~WENetwork() = default;
		virtual bool setNetworkConfig(WENetworkConfig* p_networkConfig) = 0;
		virtual const WENetworkConfig* getNetworkConfig() const = 0;

		virtual bool setNetworkSettings(const std::string & p_ipAddress, const std::string & p_netmask, const std::string & p_gateway) = 0;
		virtual bool isIPAddressValid(const std::string & p_ipAddress) = 0;

		virtual bool readNetworkSettings() = 0;
		virtual bool applyNetworkSettings() = 0;
		virtual void setNetworkinterface(const std::string & p_name) = 0;

		virtual bool setIPAddress(const std::string & p_ipAddress, bool p_apply) = 0;
		virtual bool setNetmaskIP(const std::string & p_netmask, bool p_apply) = 0;
		virtual bool setGatewayIP(const std::string & p_gateway, bool p_apply) = 0;

		virtual const std::string & getDefaultIPAdress() const = 0;
		virtual const std::string & getDefaultNetmaskIP() const = 0;
		virtual const std::string & getDefaultGatewayIP() const = 0;
	};
}//end of namespace WE

#endif /* WE_NETWORK_H_ */
