/**
 *	@file we_sleep.h
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 *
 *	@brief Platform independent sleep function(s)
 *
 *	The class WESleep provides functions to sleep a process
 *	(without interruption from signals). Millisecond
 */

#ifndef WE_SLEEP_H_
#define WE_SLEEP_H_

namespace WE
{

/**
 *	@class WESleep
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@brief Class that has a static function to lay a process to sleep
 *
 *	@namespace WE
 *
 */
class WESleep
{
public:
	WESleep() = delete;
	~WESleep() = delete;
	/**
	 *	@fn static void sleep(int p_seconds, long int p_nanoSeconds)
	 *
	 *	@brief Sleep for the specified time in seconds an nanoseconds
	 *
	 *	@author dsl@wachendorff.de
	 *	@author Dirk Sterzel
	 *
	 *	@param p_seconds sleep time in seconds
	 *	@param p_nanoSeconds sleep time in nanoseconds
	 *
	 *	@warning If this function is interrupted (e.g. by a signal)
	 *		it will continue to sleep!
	 *
	 *	@warning: If you want a specific thread to sleep, use WEThread::sleep()
	 */
	static void sleep(int p_seconds, long int p_nanoSeconds);
};
}//namespace WE

#endif /* WE_SLEEP_H_ */
