/**
 *	@file we_can_ecu.h
 *
 *	@author hm@wachendorff.de
 *	@author Hendrik Mueller
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2011
 *
 *	@brief This file provides the basic information of an ECU.
 *
 */

#ifndef WE_CAN_MESSAGE_LISTENER_H_
#define WE_CAN_MESSAGE_LISTENER_H_


namespace WE
{
struct WECANMessage;

/**
 *	@class WECANMessageListener
 *
 *	@brief Interface to be implemented if certain messages should not be handled by a stack itself.
 *
 */
class WECANMessageListener
{

public:
	virtual ~WECANMessageListener() = default;

	/**
	 *	@fn 	virtual int handleMessage( WECANMessage * p_msg )
	 *
	 *
	 *	@brief Interface to be called if a CAN message has to be handled.
	 *
	 *	@author ml@wachendorff.de
	 *	@author Michael Lenhart
	 *
	 *	@param p_msg	Pointer to the WECANMessage instance
	 *
	 *	@returns WE_SUCCESS in case of success
	 *	@returns WE_INVALID_PARAMETER_ERROR if any of the pointers provided is nullptr or p_portNo is 0
	 *	@returns WE_NOT_FOUND_ERROR if the message could not be handled (but was generally found to be correct)
	 *	@returns Any other error code the actual implementation considers useful
	 *
	 *	This method checks the content of p_msg and takes appropriate actions.
	 *	If content of p_msg is required to persist after this method terminates it should be copied.
	 *
	 *	More details have to be taken from documentation of the stack calling this interface.
	 *
	 *	@warning The stack calling this interface may reuse p_msg for the next message after this method returns.
	 */
	virtual int handleMessage( WECANMessage * p_msg ) = 0;
};
}	// End of namespace WE

#endif /* WE_CAN_MESSAGE_LISTENER_H_ */
