/**
 *	@file we_j1939_resources.h
 *
 *	@author hm@wachendorff.de
 *	@author Hendrik Mueller
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2011
 *
 *	@brief Implementation of J1939 stack specific resources container.
 *
 */

#ifndef WE_J1939_RESOURCES_H_
#define WE_J1939_RESOURCES_H_

#include "../../common/include/we_can_protocol.h"

namespace WE
{

class WETick;
class WEDEI;
class WEJ1939AddressHandler;
class WEJ1939TransportProtocolHandler;
class WEJ1939;

/**
 *	@class WEJ1939Resources
 *
 *	@brief Extends WECANProtocolResources with the J1939 specific resources
 *
 *	This class contains all resources needed by the J1939 stack. Using this class, it
 *	is easy to pass al important stuff to the statemachine too.
 *
 *	We expect them to be constant for a stack instance - so the members are also "const".
 *
 *	@see WECANProtocolResources
 */
class WEJ1939Resources : public WECANProtocolResources
{
public:
	WEJ1939AddressHandler * const addressHandler;		//!< Pointer to the address handler that takes care about address claiming
	WEJ1939TransportProtocolHandler * const tpHandler;//!< Pointer to the transport protocol that takes care about transport protocol messages
	unsigned char prefAddress;
	unsigned long long int name;			//!< The J1939 64 bit ECU name
	WEJ1939 * const j1939Instance;

	WEJ1939Resources(WEDEI * p_dei,
			 WEVariableEventDispatcher * p_varEventDispatcher,
			 unsigned int p_portNo,
			 WEJ1939AddressHandler * p_addressHandler,
			 WEJ1939TransportProtocolHandler * p_protocolHandler,
			 unsigned char p_prefAddress,
			 unsigned long long int p_name,
			 WEJ1939 * p_j1939Instance)
	:WECANProtocolResources(p_dei, p_varEventDispatcher, p_portNo),
	 addressHandler{p_addressHandler},
	 tpHandler{p_protocolHandler},
	 prefAddress{p_prefAddress},
	 name{p_name},
	 j1939Instance{p_j1939Instance}
	{
		 //nothing more to do here
	}

	virtual ~WEJ1939Resources() = default;
};
}

#endif /* WE_J1939_RESOURCES_H_ */
