/**
 *	@file we_tick.h
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 *
 *	@brief Interface for WETick class that provides the possibility to count "ticks"
 *
 *	In this file the interface for the WETick class is specified.
 */

#ifndef WE_TICK_H_
#define WE_TICK_H_

namespace WE
{


/**
 *	@class WETick
 *
 *	@author dsl@wachendorff.de
 *	@author Dirk Sterzel
 *
 *	@brief The WETick class provides an interface for counting ticks.
 *
 *	@namespace WE
 *
 *	The WETick class provides an interface for "tick" counting. One tick always represents
 *	1 millisecond.
 */
class WETick
{
public:
	/**
	 *	@fn static unsigned int getTickCount(void)
	 *
	 *	@brief This function returns the current tick count
	 *
	 *	@author dsl@wachendorff.de
	 *	@author Dirk Sterzel
	 *
	 *	@returns The current tick count as an unsigned int value (representing milliseconds)
	 */
	static unsigned int getTickCount(void);

	/**
	 *	@fn WETick()
	 *
	 *	@brief Deleted constructor
	 *
	 *	@author dsl@wachendorff.de
	 *	@author Dirk Sterzel
	 */
	WETick() = delete;
	~WETick() = delete;
};
} // namespace WE

#endif /* WE_TICK_H_ */
