/**
 *	@file helper_functions.h
 *
 *	@author satheeshkumar.p@aricent.com
 *	@author Satheeskumar Palanivel
 *
 *	@copyright ( c ) Topcon Electronics GmbH & Co. KG 2009
 *
 *	@brief string convert to convert the string between UTF8 and UTF-16
 *
 */

#ifndef WE_UTF_CONVERTER_H_
#define WE_UTF_CONVERTER_H_

#include <string>

namespace WE
{

/**
 *	@fn std::string convertToUtf8(const std::wstring& p_uft16Value)
 *
 *	@brief Function to convert the UTF-16 (wide string) type to UTF8
 *
 *	@author satheeshkumar.p@aricent.com
 *	@author Satheeskumar Palanivel
 *
 *	@param p_uft16Value UTF-16 type string
 *
 *	@returns On success returns the converted UTF8 string
 *	@returns returns empty string if failed
 */
std::string convertToUtf8(const std::wstring& p_uft16Value);

/**
 *	@fn std::wstring convertToUtf16(const std::string& p_utf8Value)
 *
 *	@brief Function to convert the UFT8 (wide string) type to UTF-16
 *
 *	@author satheeshkumar.p@aricent.com
 *	@author Satheeskumar Palanivel
 *
 *	@param p_utf8Value UTF8 type string
 *
 *	@returns On success returns the converted UTF-16 string
 *	@returns returns empty string if failed
 */
std::wstring convertToUtf16(const std::string& p_utf8Value);
}

#endif /* WE_UTF_CONVERTER_H_ */
