//socket includes
#include <sys/socket.h>
#include <sys/un.h>

#include <unistd.h> //getpid()

#include <errno.h>
#include <string.h>	//strerror

#include <QApplication>
#include <QSocketNotifier>
#include <QDebug>

#include "../../src/dei_ipc_functions.h"
#include "sockethandler.h"
#include "mainwindow.h"

// IMPORTAND NOTE:
// This is only an example. There are many things that should be done
// better (e.g. error handling after function calls)!


int main(int argc, char *argv[])
{
	QApplication a(argc, argv);
	MainWindow w;

	//define the address of this process
	std::string myAddress = "/tmp/hwd_pcl_ipc_qt";
	//append process-ID to the address so it changes at each start (if this was c++11, std::to_string() can be used instead of snprintf)
	char pid[12];
	sprintf(pid, "%d", getpid());
	myAddress += pid;

	int socketHWD = createSocketAndConnect(myAddress, "/tmp/hardwareDaemon");
	int socketPCL = createSocketAndConnect(myAddress, "/tmp/pclient_hardware_a6g2_conv");

	if (socketHWD > 0 && socketPCL > 0)
	{
		//subscribe to value changes:
		//@RTCSeconds from hardware daemon
		subscribeToVariable(socketHWD, 0x2100, 0x7, DATA_TYPE_INT, 100);
		//@ProjektTimePage from PClient
		subscribeToVariable(socketPCL, 0x2400, 0xc, DATA_TYPE_INT, 100);
		//@USBMemStatus from hardware daemon
		subscribeToVariable(socketHWD, 0x2010, 0x9, DATA_TYPE_INT, 100);
		//@USBMountPath from hardware daemon
		subscribeToVariable(socketHWD, 0x2010, 0xc, DATA_TYPE_STRING, 100);

		//The SocketHandler class will take care of sendind/receiving
		SocketHandler handler(socketHWD, socketPCL);

		//Create two QSocketNotifiers that will emit the activated signal when they are readable
		QSocketNotifier hwdNotifier(socketHWD, QSocketNotifier::Read);
		QSocketNotifier pclNotifier(socketPCL, QSocketNotifier::Read);
		//connect the socket notifiers with the socket handler
		QObject::connect(&hwdNotifier, SIGNAL(activated(int)), &handler, SLOT(socketReadable(int)));
		QObject::connect(&pclNotifier, SIGNAL(activated(int)), &handler, SLOT(socketReadable(int)));

		//The SocketHandler will receive and "parse" the incomming messages and emits signals
		//for the changed variables. Connect these signals to the slots of the UI objects to display the values.
		QObject::connect((QObject*)&handler, SIGNAL(secondsChanged(int)), (QObject*)w.getSecondsWidget(), SLOT(display(int)));
		QObject::connect((QObject*)&handler, SIGNAL(pageTimeChanged(int)), (QObject*)w.getPageTimeWidget(), SLOT(display(int)));
		QObject::connect((QObject*)&handler, SIGNAL(usbMountStateChanged(bool)), (QObject*)w.getMountCheckbox(), SLOT(setChecked(bool)));
		QObject::connect((QObject*)&handler, SIGNAL(usbMountPathChanged(QString)), (QObject*)w.getMountLabel(), SLOT(setText(QString)));

		//We also want to test the other direction:
		//If the @USBMemStatus checkbox is clicked when a stick is mounted,
		//send a value change request to hardware daemon to unmount the stick
		//(e.g. write value 0 to it)
		QObject::connect((QObject*)w.getMountCheckbox(), SIGNAL(clicked(bool)), (QObject*)&handler, SLOT(setMountState(bool)));

		//show the window
		w.show();

		//and let the application run
		return a.exec();
	}
	else
	{
		if (socketHWD <= 0)
		{
			qDebug() << "Unable to connect to hardware daemon: " << strerror(errno);
		}
		if (socketPCL <= 0)
		{
			qDebug() << "Unable to connect to PClient: " << strerror(errno);
		}
		return -1;
	}
}
