#include <unistd.h> //for close
#include <errno.h> //for errno
#include <QDebug>
#include "../../src/dei_ipc_functions.h"
#include "sockethandler.h"

SocketHandler::SocketHandler(int hwdSocket, int pclSocket, QObject *parent)
	: QObject(parent),
	  hwdSocket(hwdSocket),
	  pclSocket(pclSocket)

{

}

void SocketHandler::socketReadable(int socket)
{
	//socket is readable
	IPCMessage msg;

	int ret = receiveMessage(socket, &msg, 50);
	if (ret > 0)
	{
		IPCData dat;
		dataFromBuffer(msg.data, msg.header.dataLength, &dat);
		switch(msg.header.messageType)
		{
			case VARIABLE_UPDATE_EVENT:
				//FALLTHROUGH
			case VARIABLE_GET_RESPONSE:
				//FALLTHROUGH
			case EVENT_SUBSCRIPTION_RESPONSE:
			{
				switch(dat.index)
				{
					case 0x2010:
					{
						switch(dat.subIndex)
						{
							case 0x09:
								emit usbMountStateChanged(dat.data.intValue > 0);
							break;
							case 0x0c:
							{
								QString str = (const char*)(dat.data.stringValue.value);
								emit usbMountPathChanged(str);
							}
							break;
						}
					}
					break;
					case 0x2100:
						switch(dat.subIndex)
						{
							case 0x07:
								//qDebug() << "SECONDS: " << dat.data.intValue;
								emit secondsChanged(dat.data.intValue);
							break;
						}
					break;
					case 0x2400:
						switch(dat.subIndex)
						{
							case 0x0c:
								//qDebug() << "PAGE TIME: " << dat.data.intValue;
								emit pageTimeChanged(dat.data.intValue);
							break;
						}
					break;
				}
			}
			break;
			default:
			break;
		}
	}
	else if (ret < 0)
	{
		qDebug() << "ERROR on socket " << socket << ": " << strerror(errno);
		qDebug() << "Connection terminated!";
		close(socket);
		//you might want to implement a re-connect mechanism in this case
		//(create new socket and connect again)
	}
}

void SocketHandler::setMountState(bool state)
{
	IPCData dat;
	dat.index = 0x2010;
	dat.subIndex = 0x09;
	dat.type = DATA_TYPE_INT;
	dat.data.intValue = state ? 1 : 0;

	IPCMessage msg;
	msg.header.messageType = VARIABLE_SET_REQUEST;
	msg.header.dataLength = getBufferSize(&dat);
	msg.data = new unsigned char[msg.header.dataLength];
	memset(msg.data, 0, msg.header.dataLength);
	convertToBuffer(msg.data, &dat);
	sendMessage(hwdSocket, &msg, 50);
}
